/*

asiq.go:  to count the NGS reads uniquely mapped to allele-specific isoforms

USAGE:  ./asiq	SAMPLE_isoform_phased.txt  SAMPLE_ngs.hq_isoform.filtered.sam  >SAMPLE_asiq.out.txt
*/

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"regexp"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 2 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	//打开phasedIso_output1文件
	phasedIso_output1filename := os.Args[1]
	phasedIso_output1file, openphasedIso_output1file_Err := os.Open(phasedIso_output1filename)
	if openphasedIso_output1file_Err != nil {
		fmt.Printf("Error happened in opening %s\n", phasedIso_output1filename)
		return
	}
	defer phasedIso_output1file.Close()
	phasedIso_output1Reader := bufio.NewReader(phasedIso_output1file)
	isocountsmaps := make(map[string]int)
	iso_SNPmaps := make(map[string][]string)
	for {
		phasedIso_output1_raw, readphasedIso_output1_Err := phasedIso_output1Reader.ReadString('\n')
		phasedIso_output1_raw = seqTrim(phasedIso_output1_raw)
		if len(phasedIso_output1_raw) > 0 {
			phasedIso_output1lineblocks := strings.Split(phasedIso_output1_raw, "\t")
			isocountsmaps[phasedIso_output1lineblocks[0]] = 0
			iso_SNPmaps[phasedIso_output1lineblocks[1]] = append(iso_SNPmaps[phasedIso_output1lineblocks[1]], phasedIso_output1lineblocks[3])
		}
		if readphasedIso_output1_Err == io.EOF {
			break
		}
	}
	//打开CCS_sam_deal文件
	CCS_sam_dealfilename := os.Args[2]
	CCS_sam_dealfile, openCCS_sam_dealfile_Err := os.Open(CCS_sam_dealfilename)
	if openCCS_sam_dealfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", CCS_sam_dealfilename)
		return
	}
	defer CCS_sam_dealfile.Close()
	CCS_sam_dealReader := bufio.NewReader(CCS_sam_dealfile)

	var start_index1, end_index1, start_index2, end_index2 int
	var readNisoname_oflastline, fasta1, fasta2 string

	for {
		CCS_sam_deal_raw, readCCS_sam_deal_Err := CCS_sam_dealReader.ReadString('\n')
		CCS_sam_deal_raw = seqTrim(CCS_sam_deal_raw)
		pat, _ := regexp.MatchString("^@", CCS_sam_deal_raw)
		if pat {
			continue
		}
		if len(CCS_sam_deal_raw) > 0 {
			CCS_sam_deallineblocks := strings.Split(CCS_sam_deal_raw, "\t")
			_, ok := iso_SNPmaps[CCS_sam_deallineblocks[2]]
			if !ok {
				continue
			}
			//fmt.Println(CCS_sam_deal_raw)
			CIGARmaps := make(map[string]int)
			lenthadd := 0
			if readNisoname_oflastline != CCS_sam_deallineblocks[0]+CCS_sam_deallineblocks[2] {
				readNisoname_oflastline = CCS_sam_deallineblocks[0] + CCS_sam_deallineblocks[2]
				fasta1 = CCS_sam_deallineblocks[9]
				start_index1, _ = strconv.Atoi(CCS_sam_deallineblocks[3])
				indexchangemaps := make(map[int]string)
				for j, s := range CCS_sam_deallineblocks[5] {
					if strings.ContainsAny(string(s), "MIDNSHP=X") {
						lenthnums, _ := strconv.Atoi(CCS_sam_deallineblocks[5][lenthadd:j])
						CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
						lenthadd = j + 1
						if string(s) == "D" {
							indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] = fasta1[CIGARmaps["M"]+CIGARmaps["I"]-1 : CIGARmaps["M"]+CIGARmaps["I"]]
							for i := 1; i <= lenthnums; i++ {
								indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] = indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] + "-"
							}
						} else if string(s) == "I" {
							for i := 1; i <= lenthnums; i++ {
								indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]+i-1-lenthnums] = ""
							}
						}
					}
				}
				end_index1 = start_index1 + CIGARmaps["M"] + CIGARmaps["D"] + CIGARmaps["N"] - 1
				var fastachange string
				for i, r := range fasta1 {
					_, ok := indexchangemaps[i]
					if ok {
						fastachange = fastachange + indexchangemaps[i]
					} else if !ok {
						fastachange = fastachange + string(r)
					}
				}
				fasta1 = fastachange
				//fmt.Printf("%s\t%s\t", CCS_sam_deallineblocks[0], CCS_sam_deallineblocks[2])
			} else if readNisoname_oflastline == CCS_sam_deallineblocks[0]+CCS_sam_deallineblocks[2] { //以每两行为单位读取
				readNisoname_oflastline = CCS_sam_deallineblocks[0] + CCS_sam_deallineblocks[2]
				fasta2 = CCS_sam_deallineblocks[9]
				start_index2, _ = strconv.Atoi(CCS_sam_deallineblocks[3])
				indexchangemaps := make(map[int]string)

				for j, s := range CCS_sam_deallineblocks[5] {
					if strings.ContainsAny(string(s), "MIDNSHP=X") {
						lenthnums, _ := strconv.Atoi(CCS_sam_deallineblocks[5][lenthadd:j])
						CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
						lenthadd = j + 1

						if string(s) == "D" {
							indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] = fasta2[CIGARmaps["M"]+CIGARmaps["I"]-1 : CIGARmaps["M"]+CIGARmaps["I"]]
							for i := 1; i <= lenthnums; i++ {
								indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] = indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]-1] + "-"
							}
						} else if string(s) == "I" {
							for i := 1; i <= lenthnums; i++ {
								indexchangemaps[CIGARmaps["M"]+CIGARmaps["I"]+i-1-lenthnums] = ""
							}
						}
					}
				}

				end_index2 = start_index2 + CIGARmaps["M"] + CIGARmaps["D"] + CIGARmaps["N"] - 1
				var fastachange string
				for i, r := range fasta2 {
					_, ok := indexchangemaps[i]
					if ok {
						fastachange = fastachange + indexchangemaps[i]
					} else if !ok {
						fastachange = fastachange + string(r)
					}
				}
				fasta2 = fastachange
				//fmt.Printf("%d\t%d\t%s\n%s\t%s\t%d\t%d\t%s\n", start_index1, end_index1, fasta1, CCS_sam_deallineblocks[0], CCS_sam_deallineblocks[2], start_index2, end_index2, fasta2)

				if end_index1 >= end_index2 {
					start_index1, start_index2 = start_index2, start_index1
					end_index1, end_index2 = end_index2, end_index1
					fasta1, fasta2 = fasta2, fasta1
				}
				for _, SNPlinks := range iso_SNPmaps[CCS_sam_deallineblocks[2]] { //在上一步处理中有把长的SNP放前面，所以是先判断长的SNP
					index_SNPs := strings.Split(SNPlinks, "|")
					breakflag := 0
					for _, index_SNP := range index_SNPs {
						index_or_SNP := strings.Split(index_SNP, "_")
						index, _ := strconv.Atoi(index_or_SNP[0])
						if index <= 0 {
							continue
						}
						if index >= start_index2 && index <= end_index1 && start_index1 <= start_index2 {
							if index-start_index1+len(index_or_SNP[1]) > len(fasta1) && index-start_index2+len(index_or_SNP[1]) > len(fasta2) {
								continue
							} else if index-start_index1+len(index_or_SNP[1]) > len(fasta1) {
								SNPfromRead2 := fasta2[index-start_index2 : index-start_index2+len(index_or_SNP[1])]
								if SNPfromRead2 == index_or_SNP[1] {
									isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] = isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] + 1
									break
									breakflag = 1
								}
							} else {
								SNPfromRead1 := fasta1[index-start_index1 : index-start_index1+len(index_or_SNP[1])]
								SNPfromRead2 := fasta2[index-start_index2 : index-start_index2+len(index_or_SNP[1])]
								if SNPfromRead1 == SNPfromRead2 && SNPfromRead1 == index_or_SNP[1] {
									isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] = isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] + 1
									break
									breakflag = 1
								}
							}
						} else if index >= start_index1 && index <= end_index1 {
							if index-start_index1+len(index_or_SNP[1]) > len(fasta1) {
								continue
							}
							if fasta1[index-start_index1:index-start_index1+len(index_or_SNP[1])] == index_or_SNP[1] {
								isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] = isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] + 1
								break
								breakflag = 1
							}

						} else if index >= start_index2 && index <= end_index2 {
							if index-start_index2+len(index_or_SNP[1]) > len(fasta2) {
								continue
							}
							if fasta2[index-start_index2:index-start_index2+len(index_or_SNP[1])] == index_or_SNP[1] {
								isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] = isocountsmaps[CCS_sam_deallineblocks[2]+"_"+SNPlinks] + 1
								break
								breakflag = 1
							}
						}
					}
					if breakflag == 1 {
						break
					}
				}
			}
		}
		if readCCS_sam_deal_Err == io.EOF {
			break
		}
	}
	for isoname, frequency := range isocountsmaps {
		fmt.Printf("%s\t%d\n", isoname, frequency)
	}

}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
func reverse(s string) string {
	r := []rune(s)
	for from, to := 0, len(r)-1; from < to; from, to = from+1, to-1 {
		r[from], r[to] = r[to], r[from]
	}
	return string(r)
}

func comp(s string) string {
	s = strings.Replace(s, "A", "X", -1)
	s = strings.Replace(s, "T", "A", -1)
	s = strings.Replace(s, "X", "T", -1)
	s = strings.Replace(s, "C", "X", -1)
	s = strings.Replace(s, "G", "C", -1)
	s = strings.Replace(s, "X", "G", -1)
	s = strings.Replace(s, "a", "x", -1)
	s = strings.Replace(s, "t", "a", -1)
	s = strings.Replace(s, "x", "t", -1)
	s = strings.Replace(s, "c", "x", -1)
	s = strings.Replace(s, "g", "c", -1)
	s = strings.Replace(s, "x", "g", -1)
	return s
}
