/*

asiqFilt.go:  to screen the uniquely mapped NGS read pairs to hq_isoforms and the mapping relationship

USAGE: ./asiqFilt  SAMPLE_ngs.hq_isoform.sam   >SAMPLE_ngs.hq_isoform.filtered.sam

*/

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"regexp"
	//	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 2 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}

	//打开CCS_sam文件
	CCS_samfilename := os.Args[1]
	CCS_samfile, openCCS_samfile_Err := os.Open(CCS_samfilename)
	if openCCS_samfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", CCS_samfilename)
		return
	}
	defer CCS_samfile.Close()
	CCS_samReader := bufio.NewReader(CCS_samfile)
	CCScountmaps := make(map[string]int)
	CCS_NGScountmaps := make(map[string]int)
	for {
		CCS_sam_raw, readCCS_sam_Err := CCS_samReader.ReadString('\n')
		CCS_sam_raw = seqTrim(CCS_sam_raw)
		pat, _ := regexp.MatchString("^@", CCS_sam_raw)
		if pat {
			continue
		}
		if len(CCS_sam_raw) > 0 {
			CCS_samlineblocks := strings.Split(CCS_sam_raw, "\t")
			if CCS_samlineblocks[2] == "*" || CCS_samlineblocks[5] == "*" {
				continue
			}
			_, ok1 := CCScountmaps[CCS_samlineblocks[0]]
			_, ok2 := CCS_NGScountmaps[CCS_samlineblocks[0]+CCS_samlineblocks[2]]
			if !ok1 {
				CCScountmaps[CCS_samlineblocks[0]] = 1
			} else if ok1 {
				CCScountmaps[CCS_samlineblocks[0]] = CCScountmaps[CCS_samlineblocks[0]] + 1
			}
			if !ok2 {
				CCS_NGScountmaps[CCS_samlineblocks[0]+CCS_samlineblocks[2]] = 1
			} else if ok2 {
				CCS_NGScountmaps[CCS_samlineblocks[0]+CCS_samlineblocks[2]] = CCS_NGScountmaps[CCS_samlineblocks[0]+CCS_samlineblocks[2]] + 1
			}
		}
		if readCCS_sam_Err == io.EOF {
			break
		}
	}

	CCS_samagainfilename := os.Args[1]
	CCS_samagainfile, openCCS_samagainfile_Err := os.Open(CCS_samagainfilename)
	if openCCS_samagainfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", CCS_samagainfilename)
		return
	}
	defer CCS_samagainfile.Close()
	CCS_samagainReader := bufio.NewReader(CCS_samagainfile)

	for {
		CCS_samagain_raw, readCCS_samagain_Err := CCS_samagainReader.ReadString('\n')
		CCS_samagain_raw = seqTrim(CCS_samagain_raw)
		pat, _ := regexp.MatchString("^@", CCS_samagain_raw)
		if pat {
			continue
		}
		if len(CCS_samagain_raw) > 0 {
			CCS_samagainlineblocks := strings.Split(CCS_samagain_raw, "\t")
			if CCS_samagainlineblocks[2] == "*" || CCS_samagainlineblocks[5] == "*" {
				continue
			}
			if CCScountmaps[CCS_samagainlineblocks[0]] == 2 && CCS_NGScountmaps[CCS_samagainlineblocks[0]+CCS_samagainlineblocks[2]] == 2 {
				fmt.Println(CCS_samagain_raw)
			}
		}
		if readCCS_samagain_Err == io.EOF {
			break
		}
	}

}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
func reverse(s string) string {
	r := []rune(s)
	for from, to := 0, len(r)-1; from < to; from, to = from+1, to-1 {
		r[from], r[to] = r[to], r[from]
	}
	return string(r)
}

func comp(s string) string {
	s = strings.Replace(s, "A", "X", -1)
	s = strings.Replace(s, "T", "A", -1)
	s = strings.Replace(s, "X", "T", -1)
	s = strings.Replace(s, "C", "X", -1)
	s = strings.Replace(s, "G", "C", -1)
	s = strings.Replace(s, "X", "G", -1)
	s = strings.Replace(s, "a", "x", -1)
	s = strings.Replace(s, "t", "a", -1)
	s = strings.Replace(s, "x", "t", -1)
	s = strings.Replace(s, "c", "x", -1)
	s = strings.Replace(s, "g", "c", -1)
	s = strings.Replace(s, "x", "g", -1)
	return s
}
