//bcfCorrect.go: to convert NGS-uncovered PSEUDO_LONG_READ bases (including low-quality or high-quality SNPs) to low case
//USAGE: ./bcfCorrect	BCF_DIRECTLY_DERIVED_VCF_FILE	pseudo.longReads.fasta		>NGS.corrected.pseudo.longReads.fasta

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(BCFfile or fastafile)!\n")
		return
	}
	BCFFilename := os.Args[1]
	BCFFile, openErr1 := os.Open(BCFFilename)
	if openErr1 != nil {
		fmt.Printf("Error happened in opening %s\n", BCFFilename)
		return
	}
	defer BCFFile.Close()

	BCFReader := bufio.NewReader(BCFFile)
	BCFmaps := make(map[string][]string)
	for {
		BCFraw, readBCFErr := BCFReader.ReadString('\n')

		BCFraw = seqTrim(BCFraw)
		if len(BCFraw) > 1 {
			lineblocks := strings.Split(BCFraw, "\t")
			if strings.Contains(BCFraw, "#") || len(lineblocks) < 2 {
				continue
			}
			BCFmaps[lineblocks[0]] = append(BCFmaps[lineblocks[0]], lineblocks[1])
		}
		if readBCFErr == io.EOF {
			break
		}
	}

	fastafilename := os.Args[2]
	fastaFile, openErr2 := os.Open(fastafilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", fastafilename)
		return
	}
	defer fastaFile.Close()
	fastaReader := bufio.NewReader(fastaFile)
	var seqname string
	fastamaps := make(map[string]string)
	for {
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") {

			if len(seqname) > 1 {

				for _, index := range BCFmaps[seqname] {
					index_int, _ := strconv.Atoi(index)
					fastamaps[seqname] = fastamaps[seqname][:index_int-1] + strings.ToUpper(fastamaps[seqname][index_int-1:index_int]) + fastamaps[seqname][index_int:]
				}
				fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
			}

			seqname = fastaraw[1:]

		} else if len(fastaraw) > 0 {
			fastaraw = strings.ToLower(fastaraw)
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}

	}
	if len(seqname) > 1 {

		for _, index := range BCFmaps[seqname] {
			index_int, _ := strconv.Atoi(index)
			fastamaps[seqname] = fastamaps[seqname][:index_int-1] + strings.ToUpper(fastamaps[seqname][index_int-1:index_int]) + fastamaps[seqname][index_int:]
		}
		fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
	}
}
func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")

	return s
}
