//lqMark.go: to convert the low-quality SNP sites of 'NGS.corrected.pseudo.longReads.fasta' into lowcase
//USAGE: ./lqMark	FILTERED_VCF_FILE	NGS.corrected.pseudo.longReads.fasta	>lq_marked.NGS_corrected.pseudo.longReads.fasta

//只对第二列的lowqual位点替换成小写，不管第四列的长度

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	VCFFilename := os.Args[1]
	VCFFile, openErr1 := os.Open(VCFFilename)
	if openErr1 != nil {
		fmt.Printf("Error happened in opening %s\n", VCFFilename)
		return
	}
	defer VCFFile.Close()

	VCFReader := bufio.NewReader(VCFFile)
	VCFmaps := make(map[string][]string)
	for {
		VCFraw, readVCFErr := VCFReader.ReadString('\n')
		VCFraw = seqTrim(VCFraw)
		if len(VCFraw) > 1 {
			lineblocks := strings.Split(VCFraw, "\t")
			if strings.Contains(VCFraw, "#") || len(lineblocks) < 2 {
				continue
			}
			if strings.Contains(VCFraw, "LOWQUAL") {
				info := lineblocks[1] + ";" + strconv.Itoa(len(lineblocks[3])) + ";" + "0"
				VCFmaps[lineblocks[0]] = append(VCFmaps[lineblocks[0]], info)
			}

		}
		if readVCFErr == io.EOF {
			break
		}
	}

	fastafilename := os.Args[2]
	fastaFile, openErr2 := os.Open(fastafilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", fastafilename)
		return
	}
	defer fastaFile.Close()
	fastaReader := bufio.NewReader(fastaFile)
	var seqname string
	fastamaps := make(map[string]string)

	for {
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") {

			if len(seqname) > 1 {

				for _, info := range VCFmaps[seqname] {
					infoblocks := strings.Split(info, ";")
					if infoblocks[2] == "0" {
						index_int, _ := strconv.Atoi(infoblocks[0])
						len_int := 1
						fastamaps[seqname] = fastamaps[seqname][:index_int-1] + strings.ToLower(fastamaps[seqname][index_int-1:index_int-1+len_int]) + fastamaps[seqname][index_int-1+len_int:]
					}
				}
				fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
			}
			seqname = fastaraw[1:]
		} else if len(fastaraw) > 0 {
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}
	}

	if len(seqname) > 1 {
		for _, info := range VCFmaps[seqname] {
			infoblocks := strings.Split(info, ";")
			if infoblocks[2] == "0" {
				index_int, _ := strconv.Atoi(infoblocks[0])
				len_int := 1
				fastamaps[seqname] = fastamaps[seqname][:index_int-1] + strings.ToLower(fastamaps[seqname][index_int-1:index_int-1+len_int]) + fastamaps[seqname][index_int-1+len_int:]
			}
		}
		fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
	}
}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
