//modifiedLongReadsCorrect.go: to correct modified raw long reads according to 'SNP_marked.NGS_corrected.pseudo.longReads.fasta'

//USAGE: ./modifiedLongReadsCorrect		pseudo.longReads.coord.txt	modified.longReads.fasta	FILTER_VCF	SNP_marked.NGS_corrected.pseudo.longReads.fasta		
//AUTOMATIC OUTPUT: SNP_marked.NGS_corrected.modified.longReads.fasta; modified.longReads.SNP.annotation.txt


//refer to 'modifiedLongReadsCorrect.txt'

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 4 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	output1file, _ := os.Create("SNP_marked.NGS_corrected.modified.longReads.fasta") //创建fasta输出文件
	defer output1file.Close()
	output2file, _ := os.Create("modified.longReads.SNP.annotation.txt") //创建表格输出文件
	defer output2file.Close()

	sam_deal_2filename := os.Args[1]
	sam_deal_2file, opensam_dealErr := os.Open(sam_deal_2filename)
	if opensam_dealErr != nil {
		fmt.Printf("Error happened in opening %s\n", sam_deal_2filename)
		return
	}
	defer sam_deal_2file.Close()
	sam_dealReader := bufio.NewReader(sam_deal_2file)
	SAM_CIGARmaps := make(map[string]string)
	chr_maps := make(map[string]string)
	chrindex_maps := make(map[string]int)
	for {
		sam_dealraw, readsam_dealErr := sam_dealReader.ReadString('\n')
		sam_dealraw = seqTrim(sam_dealraw)
		if len(sam_dealraw) > 0 {
			samlineblocks := strings.Split(sam_dealraw, "\t")
			SAM_CIGARmaps[samlineblocks[0]] = samlineblocks[3]
			chr_maps[samlineblocks[0]] = samlineblocks[1]
			chrindex, _ := strconv.Atoi(samlineblocks[2])
			chrindex = chrindex - 1
			chrindex_maps[samlineblocks[0]] = chrindex
		}
		if readsam_dealErr == io.EOF {
			break
		}
	}
	fmt.Println("SAM_deal_2 读取结束", len(SAM_CIGARmaps))

	RAW_SAMfilename := os.Args[2]
	RAW_SAMfile, openRAW_SAMErr := os.Open(RAW_SAMfilename)
	if openRAW_SAMErr != nil {
		fmt.Printf("Error happened in opening %s\n", RAW_SAMfilename)
		return
	}
	defer RAW_SAMfile.Close()
	RAW_SAMReader := bufio.NewReader(RAW_SAMfile)
	var RAW_seqname, RAW_seq string
	RAW_fastamaps := make(map[string]string)
	for {
		RAW_SAMraw, readRAW_SAMErr := RAW_SAMReader.ReadString('\n')
		RAW_SAMraw = seqTrim(RAW_SAMraw)
		pat := strings.Contains(RAW_SAMraw, ">")
		if pat && len(RAW_seq) >= 1 {
			_, ok := SAM_CIGARmaps[RAW_seqname]
			if ok {
				RAW_fastamaps[RAW_seqname] = RAW_seq
			}
			RAW_seq = ""
		}
		if pat {
			RAW_seqname = RAW_SAMraw[1:]

		}
		if !pat {
			RAW_seq = RAW_seq + RAW_SAMraw
		}

		if readRAW_SAMErr == io.EOF {
			_, ok := SAM_CIGARmaps[RAW_seqname] //判断最后一条序列
			if ok {
				RAW_fastamaps[RAW_seqname] = RAW_seq
			}
			break
		}
	}
	fmt.Println("RAW_SAM 读取结束", len(SAM_CIGARmaps))

	VCFfilename := os.Args[3]
	VCFfile, openVCFErr := os.Open(VCFfilename)
	if openVCFErr != nil {
		fmt.Printf("Error happened in opening %s\n", VCFfilename)
		return
	}
	defer VCFfile.Close()
	VCFReader := bufio.NewReader(VCFfile)
	VCFmaps := make(map[string][]string)
	for {
		VCFraw, readVCFErr := VCFReader.ReadString('\n')
		VCFraw = seqTrim(VCFraw)
		if len(VCFraw) > 1 {
			VCFlineblocks := strings.Split(VCFraw, "\t")
			if strings.Contains(VCFraw, "#") || len(VCFlineblocks) < 2 || strings.Contains(VCFraw, "LOWQUAL") {
				continue
			}
			info := VCFlineblocks[1] + "\t" + VCFlineblocks[3] + "\t" + VCFlineblocks[4]
			VCFmaps[VCFlineblocks[0]] = append(VCFmaps[VCFlineblocks[0]], info)
		}
		if readVCFErr == io.EOF {
			break
		}

	}

	fastafilename := os.Args[4]
	fastaFile, openErr2 := os.Open(fastafilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", fastafilename)
		return
	}
	defer fastaFile.Close()
	fastaReader := bufio.NewReader(fastaFile)
	var seqname string
	fastamaps := make(map[string]string)
	fmt.Println("VCF 读取结束")
	for {

		index_changemaps := make(map[int]string)
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") || readfastaErr == io.EOF {
			if len(seqname) >= 1 {
				pat1 := strings.Contains(fastamaps[seqname], "1")
				patX := strings.Contains(fastamaps[seqname], "X")
				patLOW := strings.ContainsAny(fastamaps[seqname], "actg")
				//a11) 无1、无X、无小写字母情况
				if !pat1 && !patLOW && !patX {
					_, output1err := output1file.WriteString(">" + seqname + "\n" + fastamaps[seqname] + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					_, output2err := output2file.WriteString(seqname + "\ta11\t*\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

				//只含小写字母部分
				if !pat1 && patLOW && !patX {
					for i, r := range fastamaps[seqname] {
						if strings.ContainsAny(string(r), "atcg") {
							CIGARmaps := make(map[string]int)
							lenthadd := 0
							flag := 0
							for j, s := range SAM_CIGARmaps[seqname] {
								if strings.ContainsAny(string(s), "MIDNSHP=X") {
									lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
									CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
									lenthadd = j + 1
									if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										break
									} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										flag = 1
										continue
									}
									if flag == 1 {
										if string(s) == "I" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
											index_changemaps[i] = index_changemaps[i] + strings.ToLower(RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums])
										}
										break
									}
								}
							}
						}
					}
					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					_, output2err := output2file.WriteString(seqname + "\ta12\t*\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

				//只含X部分
				if !pat1 && !patLOW && patX {
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "X") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {
									if len(lineblocks[1]) <= len(lineblocks[2]) {
										index_changemaps[i] = lineblocks[2][:len(lineblocks[2])-len(lineblocks[1])+1]

									} else if len(lineblocks[1]) > len(lineblocks[2]) {
										//fmt.Println(seqname)
										index_changemaps[i] = lineblocks[2][:1]

										for j := 1; j <= len(lineblocks[1])-len(lineblocks[2]); j++ {
											index_changemaps[i+j] = "-"
										}
										i = i + len(lineblocks[1]) - len(lineblocks[2])
									}
									break
								}
							}
						}
					}
					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					_, output2err := output2file.WriteString(seqname + "\ta21\t*\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

				//含X和小写部分
				if !pat1 && patLOW && patX {
					for i, r := range fastamaps[seqname] {
						if strings.ContainsAny(string(r), "atcg") {
							CIGARmaps := make(map[string]int)
							lenthadd := 0
							flag := 0
							for j, s := range SAM_CIGARmaps[seqname] {
								if strings.ContainsAny(string(s), "MIDNSHP=X") {
									lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
									CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
									lenthadd = j + 1
									if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										break
									} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										flag = 1
										continue
									}
									if flag == 1 {
										if string(s) == "I" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
											index_changemaps[i] = index_changemaps[i] + strings.ToLower(RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums])
										}
										break
									}
								}
							}
						}
					}

					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "X") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {
									if len(lineblocks[1]) <= len(lineblocks[2]) {
										index_changemaps[i] = lineblocks[2][:len(lineblocks[2])-len(lineblocks[1])+1]

									} else if len(lineblocks[1]) > len(lineblocks[2]) {
										//fmt.Println(seqname)
										index_changemaps[i] = lineblocks[2][:1]

										for j := 1; j <= len(lineblocks[1])-len(lineblocks[2]); j++ {
											index_changemaps[i+j] = "-"
										}
										i = i + len(lineblocks[1]) - len(lineblocks[2])
									}
									break
								}
							}
						}
					}
					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					_, output2err := output2file.WriteString(seqname + "\ta22\t*\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

				}

				//只含1部分
				if pat1 && !patLOW && !patX {
					var output2string []string
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "1") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {

									//AC=1并且两个长度都是1的情况
									if len(lineblocks[1]) == len(lineblocks[2]) && len(lineblocks[1]) == 1 {
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")

													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")
														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")
														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														lens := len(index_changemaps[i])
														index_changemaps[i] = index_changemaps[i][1:lens-1] + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
														output2string[len(output2string)-1] = chr_maps[seqname] + "_" + strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"]) + "_" + index_changemaps[i] + "_" + "0"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
													}
													break
												}
											}
										}
									}
									//AC=1并且两个长度不相等的情况
									if !strings.Contains(lineblocks[2], ",") && len(lineblocks[1]) != len(lineblocks[2]) {
										var SNP1, SNP2 string
										if lineblocks[1] > lineblocks[2] {
											SNP1 = lineblocks[1]
											SNP2 = lineblocks[2]
										} else {
											SNP1 = lineblocks[2]
											SNP2 = lineblocks[1]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")

										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
									//AC=1,1的情况
									if strings.Contains(lineblocks[2], ",") {
										SNPblocks := strings.Split(lineblocks[2], ",")
										var SNP1, SNP2 string
										if SNPblocks[0] > SNPblocks[1] {
											SNP1 = SNPblocks[0]
											SNP2 = SNPblocks[1]
										} else {
											SNP1 = SNPblocks[1]
											SNP2 = SNPblocks[0]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")
										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
								}
							}

						}
					}
					_, output2err := output2file.WriteString(seqname + "\tb11\t")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, s := range output2string {
						if i > 0 {
							_, output2err := output2file.WriteString("|")
							if output2err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						_, output2err := output2file.WriteString(s)
						if output2err == io.EOF {
							fmt.Println("写入错误")
							return
						}
					}
					_, output2err = output2file.WriteString("\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

				//b12) 有1、无X、有小写字母 -->
				if pat1 && patLOW && !patX {
					//循环小写字母
					for i, r := range fastamaps[seqname] {
						if strings.ContainsAny(string(r), "atcg") {
							CIGARmaps := make(map[string]int)
							lenthadd := 0
							flag := 0
							for j, s := range SAM_CIGARmaps[seqname] {
								if strings.ContainsAny(string(s), "MIDNSHP=X") {
									lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
									CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
									lenthadd = j + 1
									if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										break
									} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										flag = 1
										continue
									}
									if flag == 1 {
										if string(s) == "I" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
											index_changemaps[i] = index_changemaps[i] + strings.ToLower(RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums])
										}
										break
									}
								}
							}
						}
					}
					//循环1  SNP的
					var output2string []string
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "1") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {

									//AC=1并且两个长度都是1的情况
									if len(lineblocks[1]) == len(lineblocks[2]) && len(lineblocks[1]) == 1 {
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")

													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")

														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")
														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														lens := len(index_changemaps[i])
														index_changemaps[i] = index_changemaps[i][1:lens-1] + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
														output2string[len(output2string)-1] = chr_maps[seqname] + "_" + strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"]) + "_" + index_changemaps[i] + "_" + "0"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
													}
													break

												}
											}
										}
									}
									//AC=1并且两个长度不相等的情况
									if !strings.Contains(lineblocks[2], ",") && len(lineblocks[1]) != len(lineblocks[2]) {
										var SNP1, SNP2 string
										if lineblocks[1] > lineblocks[2] {
											SNP1 = lineblocks[1]
											SNP2 = lineblocks[2]
										} else {
											SNP1 = lineblocks[2]
											SNP2 = lineblocks[1]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")

										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
									//AC=1,1的情况
									if strings.Contains(lineblocks[2], ",") {
										SNPblocks := strings.Split(lineblocks[2], ",")
										var SNP1, SNP2 string
										if SNPblocks[0] > SNPblocks[1] {
											SNP1 = SNPblocks[0]
											SNP2 = SNPblocks[1]
										} else {
											SNP1 = SNPblocks[1]
											SNP2 = SNPblocks[0]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = index_changemaps[i] + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")
										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
								}
							}

						}
					}
					_, output2err := output2file.WriteString(seqname + "\tb12\t")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, s := range output2string {
						if i > 0 {
							_, output2err := output2file.WriteString("|")
							if output2err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						_, output2err := output2file.WriteString(s)
						if output2err == io.EOF {
							fmt.Println("写入错误")
							return
						}
					}
					_, output2err = output2file.WriteString("\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

				// b21) 有1、有X、无小写字母 -->
				if pat1 && !patLOW && patX {
					//先循环X的
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "X") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {
									if len(lineblocks[1]) <= len(lineblocks[2]) {
										index_changemaps[i] = lineblocks[2][:len(lineblocks[2])-len(lineblocks[1])+1]

									} else if len(lineblocks[1]) > len(lineblocks[2]) {
										//fmt.Println(seqname)
										index_changemaps[i] = lineblocks[2][:1]

										for j := 1; j <= len(lineblocks[1])-len(lineblocks[2]); j++ {
											index_changemaps[i+j] = "-"
										}
										i = i + len(lineblocks[1]) - len(lineblocks[2])
									}
									break
								}
							}
						}
					}
					//再循环1 SNP的
					var output2string []string
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "1") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {

									//AC=1并且两个长度都是1的情况
									if len(lineblocks[1]) == len(lineblocks[2]) && len(lineblocks[1]) == 1 {
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")

													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")

														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")
														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														lens := len(index_changemaps[i])
														index_changemaps[i] = index_changemaps[i][1:lens-1] + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
														output2string[len(output2string)-1] = chr_maps[seqname] + "_" + strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"]) + "_" + index_changemaps[i] + "_" + "0"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
													}
													break

												}
											}
										}
									}
									//AC=1并且两个长度不相等的情况
									if !strings.Contains(lineblocks[2], ",") && len(lineblocks[1]) != len(lineblocks[2]) {
										var SNP1, SNP2 string
										if lineblocks[1] > lineblocks[2] {
											SNP1 = lineblocks[1]
											SNP2 = lineblocks[2]
										} else {
											SNP1 = lineblocks[2]
											SNP2 = lineblocks[1]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")

										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
									//AC=1,1的情况
									if strings.Contains(lineblocks[2], ",") {
										SNPblocks := strings.Split(lineblocks[2], ",")
										var SNP1, SNP2 string
										if SNPblocks[0] > SNPblocks[1] {
											SNP1 = SNPblocks[0]
											SNP2 = SNPblocks[1]
										} else {
											SNP1 = SNPblocks[1]
											SNP2 = SNPblocks[0]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")
										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
								}
							}

						}
					}
					_, output2err := output2file.WriteString(seqname + "\tb21\t")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, s := range output2string {
						if i > 0 {
							_, output2err := output2file.WriteString("|")
							if output2err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						_, output2err := output2file.WriteString(s)
						if output2err == io.EOF {
							fmt.Println("写入错误")
							return
						}
					}
					_, output2err = output2file.WriteString("\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

				}

				// b22) 有1、有X、有小写字母 -->
				if pat1 && patLOW && patX {
					//循环小写字母的
					for i, r := range fastamaps[seqname] {
						if strings.ContainsAny(string(r), "atcg") {
							CIGARmaps := make(map[string]int)
							lenthadd := 0
							flag := 0
							for j, s := range SAM_CIGARmaps[seqname] {
								if strings.ContainsAny(string(s), "MIDNSHP=X") {
									lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
									CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
									lenthadd = j + 1
									if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										break
									} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
										if string(s) == "D" {
											index_changemaps[i] = "-"
										} else if string(s) != "D" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
											index_changemaps[i] = strings.ToLower(RAW_fastamaps[seqname][RAW_index : RAW_index+1])
											//fmt.Println(RAW_index)
										}
										flag = 1
										continue
									}
									if flag == 1 {
										if string(s) == "I" {
											RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
											index_changemaps[i] = index_changemaps[i] + strings.ToLower(RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums])
										}
										break
									}
								}
							}
						}
					}
					//循环X的
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "X") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {
									if len(lineblocks[1]) <= len(lineblocks[2]) {
										index_changemaps[i] = lineblocks[2][:len(lineblocks[2])-len(lineblocks[1])+1]

									} else if len(lineblocks[1]) > len(lineblocks[2]) {
										//fmt.Println(seqname)
										index_changemaps[i] = lineblocks[2][:1]

										for j := 1; j <= len(lineblocks[1])-len(lineblocks[2]); j++ {
											index_changemaps[i+j] = "-"
										}
										i = i + len(lineblocks[1]) - len(lineblocks[2])
									}
									break
								}
							}
						}
					}
					//循环1 SNP的
					var output2string []string
					for i := 0; i < len(fastamaps[seqname]); i++ {
						if strings.ContainsAny(fastamaps[seqname][i:i+1], "1") {
							for _, VCFline := range VCFmaps[seqname] {
								lineblocks := strings.Split(VCFline, "\t")
								index, _ := strconv.Atoi(lineblocks[0])
								if i == index-1 {

									//AC=1并且两个长度都是1的情况
									if len(lineblocks[1]) == len(lineblocks[2]) && len(lineblocks[1]) == 1 {
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")

													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")

														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														index_changemaps[i] = "-"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
														output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+"-"+"_"+"0")
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														if lineblocks[1] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] || lineblocks[2] == RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"1")
														} else if lineblocks[1] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] && lineblocks[2] != RAW_fastamaps[seqname][RAW_index:RAW_index+1] {
															index_changemaps[i] = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
															index_changemaps[i] = "[" + index_changemaps[i] + "]"
															output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"])+"_"+RAW_fastamaps[seqname][RAW_index:RAW_index+1]+"_"+"0")
														}
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														lens := len(index_changemaps[i])
														index_changemaps[i] = index_changemaps[i][1:lens-1] + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
														output2string[len(output2string)-1] = chr_maps[seqname] + "_" + strconv.Itoa(i+1+chrindex_maps[seqname]+CIGARmaps["N"]) + "_" + index_changemaps[i] + "_" + "0"
														index_changemaps[i] = "[" + index_changemaps[i] + "]"
													}
													break

												}
											}
										}
									}
									//AC=1并且两个长度不相等的情况
									if !strings.Contains(lineblocks[2], ",") && len(lineblocks[1]) != len(lineblocks[2]) {
										var SNP1, SNP2 string
										if lineblocks[1] > lineblocks[2] {
											SNP1 = lineblocks[1]
											SNP2 = lineblocks[2]
										} else {
											SNP1 = lineblocks[2]
											SNP2 = lineblocks[1]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")

										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
									//AC=1,1的情况
									if strings.Contains(lineblocks[2], ",") {
										SNPblocks := strings.Split(lineblocks[2], ",")
										var SNP1, SNP2 string
										if SNPblocks[0] > SNPblocks[1] {
											SNP1 = SNPblocks[0]
											SNP2 = SNPblocks[1]
										} else {
											SNP1 = SNPblocks[1]
											SNP2 = SNPblocks[0]
										}
										//原始三代SNP位置序列
										var RAW_SNP string
										CIGARmaps := make(map[string]int)
										lenthadd := 0
										flag := 0
										for j, s := range SAM_CIGARmaps[seqname] {
											if strings.ContainsAny(string(s), "MIDNSHP=X") {
												lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
												CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
												lenthadd = j + 1
												if CIGARmaps["M"]+CIGARmaps["D"] > i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													break
												} else if CIGARmaps["M"]+CIGARmaps["D"] == i+1 && flag == 0 {
													if string(s) == "D" {
														RAW_SNP = "-"
													} else if string(s) != "D" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														RAW_SNP = RAW_fastamaps[seqname][RAW_index : RAW_index+1]
													}
													flag = 1
													continue
												}
												if flag == 1 {
													if string(s) == "I" {
														RAW_index := i + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"] - lenthnums + 1
													//	fmt.Println(RAW_SNP)
														RAW_SNP = RAW_SNP + RAW_fastamaps[seqname][RAW_index:RAW_index+lenthnums]
													//	fmt.Println(RAW_SNP)

													}
													if string(s) == "N" {
														CIGARmaps["N"] = CIGARmaps["N"] - lenthnums
													}
													break
												}
											}
										}
										index := i + 1 + chrindex_maps[seqname] + CIGARmaps["N"]

										//先循环SNP1的
										var RAWSAM_SNP1 string
										lastindex := -1
										for o := i; o < i+len(SNP1); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP1 = RAWSAM_SNP1 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex == -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][RAW_index:RAW_index+1])
															} else if lastindex != -1 {
																RAWSAM_SNP1 = RAWSAM_SNP1 + RAW_fastamaps[seqname][lastindex+1:RAW_index+1]
																//fmt.Println(lastindex, RAW_fastamaps[seqname][lastindex+1:RAW_index+1])
															}
														}
														lastindex = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//fmt.Println(RAWSAM_SNP1)

										//再循环SNP2的
										var RAWSAM_SNP2 string
										lastindex2 := -1
										for o := i; o < i+len(SNP2); o++ {
											CIGARmaps := make(map[string]int)
											lenthadd := 0
											for j, s := range SAM_CIGARmaps[seqname] {
												if strings.ContainsAny(string(s), "MIDNSHP=X") {
													lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[seqname][lenthadd:j])
													CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
													lenthadd = j + 1
													if CIGARmaps["M"]+CIGARmaps["D"] >= o+1 {
														if string(s) == "D" {
															RAWSAM_SNP2 = RAWSAM_SNP2 + "-"
														} else if string(s) != "D" {
															RAW_index := o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
															if lastindex2 == -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][RAW_index:RAW_index+1]
															} else if lastindex2 != -1 {
																RAWSAM_SNP2 = RAWSAM_SNP2 + RAW_fastamaps[seqname][lastindex2+1:RAW_index+1]
															}
														}
														lastindex2 = o + CIGARmaps["S"] + CIGARmaps["I"] - CIGARmaps["D"]
														break
													}
												}
											}
										}
										//判断是否与SNP1、SNP2相等
										if RAWSAM_SNP1 == SNP1 {
											index_changemaps[i] = SNP1
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP1+"_"+"1")
										} else if RAWSAM_SNP2 == SNP2 {
											index_changemaps[i] = SNP2
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+SNP2+"_"+"1")
										} else if RAWSAM_SNP1 != SNP1 && RAWSAM_SNP2 != SNP2 {
											index_changemaps[i] = RAW_SNP
											index_changemaps[i] = "[" + index_changemaps[i] + "]"
											output2string = append(output2string, chr_maps[seqname]+"_"+strconv.Itoa(index)+"_"+RAW_SNP+"_"+"0")
										}

									}
								}
							}

						}
					}
					_, output2err := output2file.WriteString(seqname + "\tb22\t")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					for i, s := range output2string {
						if i > 0 {
							_, output2err := output2file.WriteString("|")
							if output2err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						_, output2err := output2file.WriteString(s)
						if output2err == io.EOF {
							fmt.Println("写入错误")
							return
						}
					}
					_, output2err = output2file.WriteString("\n")
					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					_, output1err := output1file.WriteString(">" + seqname + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}

					for i, r := range fastamaps[seqname] {
						_, ok := index_changemaps[i]
						if ok {
							_, output1err := output1file.WriteString(index_changemaps[i])
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						} else if !ok {
							_, output1err := output1file.WriteString(string(r))
							if output1err == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, output1err = output1file.WriteString("\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

			}
			if len(fastaraw) > 0 && strings.Contains(fastaraw, ">") {
				seqname = fastaraw[1:]
			}
		} else if len(fastaraw) > 0 {
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}
	}

}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
