/*

phasedIso.go: to identify the hq_isoforms with phased SNPs, generating two files - 'isoform_phased.txt' and 'phased.hq_isoform.fasta'.

USAGE： ./phasedIso  SAMPLE_chr_index_SNP.txt  HQ_ISOFORM_FQ  SAMPLE_phased_raw.txt  SAMPLE_HQ_ISOFORM_transcripts.txt

*/

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"regexp"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	//打开chr_index_SNP文件
	chr_index_SNPfilename := os.Args[1]
	chr_index_SNPfile, openchr_index_SNPfile_Err := os.Open(chr_index_SNPfilename)
	if openchr_index_SNPfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", chr_index_SNPfilename)
		return
	}
	defer chr_index_SNPfile.Close()
	chr_index_SNPReader := bufio.NewReader(chr_index_SNPfile)
	chr_index_SNPmaps := make(map[string]string)
	for {
		chr_index_SNP_raw, readchr_index_SNP_Err := chr_index_SNPReader.ReadString('\n')
		chr_index_SNP_raw = seqTrim(chr_index_SNP_raw)
		if len(chr_index_SNP_raw) > 0 {
			chr_index_SNPlineblocks := strings.Split(chr_index_SNP_raw, "\t")
			chr_index_SNPmaps[chr_index_SNPlineblocks[0]] = chr_index_SNPlineblocks[1]
		}
		if readchr_index_SNP_Err == io.EOF {
			break
		}
	}
	//打开fastq文件
	fastqfilename := os.Args[2]
	fastqfile, openfastqfile_Err := os.Open(fastqfilename)
	if openfastqfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", fastqfilename)
		return
	}
	defer fastqfile.Close()
	fastqReader := bufio.NewReader(fastqfile)
	Isofastamaps := make(map[string]string)
	flag := 0
	var seqname string
	var namestring []string
	for {
		fastq, readfastq_Err := fastqReader.ReadString('\n')
		fastq = seqTrim(fastq)
		if len(fastq) > 0 {
			flag++
			if flag == 1 {
				fastqlineblocks := strings.Split(fastq, " ")
				seqname = fastqlineblocks[0][1:]
				namestring = append(namestring, seqname)
			}
			if flag == 2 {
				Isofastamaps[seqname] = fastq
			}
			if flag == 4 {
				flag = 0
			}
		}
		if readfastq_Err == io.EOF {
			break
		}
	}
	//打开phase_raw文件（既所有SNP两两连锁关系的文件）
	phased_rawfilename := os.Args[3]
	phased_rawfile, openphased_rawfile_Err := os.Open(phased_rawfilename)
	if openphased_rawfile_Err != nil {
		fmt.Printf("Error happened in opening %s\n", phased_rawfilename)
		return
	}
	defer phased_rawfile.Close()
	phased_rawReader := bufio.NewReader(phased_rawfile)
	phased_rawmaps := make(map[string][]string)
	for {
		phased_raw, readphased_raw_Err := phased_rawReader.ReadString('\n')
		phased_raw = seqTrim(phased_raw)
		if len(phased_raw) > 0 {
			lineblocks := strings.Split(phased_raw, "\t")
			SNPblocks := strings.Split(lineblocks[1], ",")
			for _, SNPblock := range SNPblocks {
				phased_rawmaps[lineblocks[0]] = append(phased_rawmaps[lineblocks[0]], SNPblock)
			}
		}
		if readphased_raw_Err == io.EOF {
			break
		}
	}

	//打开sam_deal_1文件
	sam_deal_2filename := os.Args[4]
	sam_deal_2file, opensam_deal_2file_Err := os.Open(sam_deal_2filename)
	if opensam_deal_2file_Err != nil {
		fmt.Printf("Error happened in opening %s\n", sam_deal_2filename)
		return
	}
	defer sam_deal_2file.Close()
	sam_deal_2Reader := bufio.NewReader(sam_deal_2file)

	output1file, _ := os.Create("isoform_phased.txt") //创建txt输出文件
	defer output1file.Close()
	output2file, _ := os.Create("phased.hq_isoform.fasta") //创建fasta输出文件
	defer output2file.Close()

	for {
		sam_deal_2_raw, readsam_deal_2_Err := sam_deal_2Reader.ReadString('\n')
		sam_deal_2_raw = seqTrim(sam_deal_2_raw)

		pat, _ := regexp.MatchString("^@", sam_deal_2_raw)
		if pat {
			continue
		}
		CIGARmaps1 := make(map[string]int)     //为了求一条isoform对应染色体起始和结束的位置
		Iso_chrmaps := make(map[string]string) //存每条isoform的SNP对应的染色体_染色体位置
		if len(sam_deal_2_raw) > 0 {
			sam_deal_2lineblocks := strings.Split(sam_deal_2_raw, "\t")
			lenthadd := 0
			for j, s := range sam_deal_2lineblocks[5] {
				if strings.ContainsAny(string(s), "MIDNSHP=X") {
					lenthnums, _ := strconv.Atoi(sam_deal_2lineblocks[5][lenthadd:j])
					CIGARmaps1[string(s)] = CIGARmaps1[string(s)] + lenthnums
					lenthadd = j + 1
				}
			}
			start_index, _ := strconv.Atoi(sam_deal_2lineblocks[3])
			end_index := start_index + CIGARmaps1["M"] + CIGARmaps1["D"] + CIGARmaps1["N"] - 1
			for i := start_index; i <= end_index; i++ {
				_, ok := chr_index_SNPmaps[sam_deal_2lineblocks[2]+"_"+strconv.Itoa(i)]
				if ok {
					Iso_chrmaps[sam_deal_2lineblocks[2]+"_"+strconv.Itoa(i)] = "0"
				}
			}
			indexofisomaps := make(map[string]int)
			if len(Iso_chrmaps) > 0 { //根据SNP在染色体上的位置求出其在iso上的位置
				for chr_indexofchr, _ := range Iso_chrmaps {
					indexofchrs := strings.Split(chr_indexofchr, "_")
					indexofchr, _ := strconv.Atoi(indexofchrs[1])
					CIGARmaps2 := make(map[string]int) //为了求一条isoform有SNP，该SNP从染色体位置求在isoform的位置
					lenthadd := 0
					for j, s := range sam_deal_2lineblocks[5] {
						if strings.ContainsAny(string(s), "MIDNSHP=X") {
							lenthnums, _ := strconv.Atoi(sam_deal_2lineblocks[5][lenthadd:j])
							CIGARmaps2[string(s)] = CIGARmaps2[string(s)] + lenthnums
							lenthadd = j + 1
							if CIGARmaps2["M"]+CIGARmaps2["D"]+CIGARmaps2["N"] >= indexofchr-start_index+1 {
								if string(s) == "D" {
									indexofisomaps[chr_indexofchr] = -1
								} else if string(s) == "M" {
									indexofisomaps[chr_indexofchr] = indexofchr - start_index + 1 + CIGARmaps2["S"] + CIGARmaps2["I"] - CIGARmaps2["D"] - CIGARmaps2["N"]
									if sam_deal_2lineblocks[1] == "16" {
										indexofisomaps[chr_indexofchr] = len(Isofastamaps[sam_deal_2lineblocks[0]]) - indexofisomaps[chr_indexofchr] + 1
									}
								} else if string(s) == "N" {
									delete(Iso_chrmaps, chr_indexofchr)
								}
								break
							}
						}
					}
				}
			}
			SNP1maps := make(map[string]string)
			SNP2maps := make(map[string]string)
			//			L_RAW_SNPmaps := make(map[string]string)
			//			S_RAW_SNPmaps := make(map[string]string)
			for chr_indexofchr, _ := range Iso_chrmaps {
				SNPs := strings.Split(chr_index_SNPmaps[chr_indexofchr], ",")
				if len(SNPs) > 2 {
					delete(Iso_chrmaps, chr_indexofchr)
				} else if len(SNPs) == 2 {
					if len(SNPs[0]) >= len(SNPs[1]) {
						SNP1maps[chr_indexofchr] = SNPs[0]
						SNP2maps[chr_indexofchr] = SNPs[1]
						if sam_deal_2lineblocks[1] == "16" {
							SNP1maps[chr_indexofchr] = reverse(SNP1maps[chr_indexofchr])
							SNP1maps[chr_indexofchr] = comp(SNP1maps[chr_indexofchr])
							SNP2maps[chr_indexofchr] = reverse(SNP2maps[chr_indexofchr])
							SNP2maps[chr_indexofchr] = comp(SNP2maps[chr_indexofchr])
						}
					} else {
						SNP1maps[chr_indexofchr] = SNPs[1]
						SNP2maps[chr_indexofchr] = SNPs[0]
						if sam_deal_2lineblocks[1] == "16" {
							SNP1maps[chr_indexofchr] = reverse(SNP1maps[chr_indexofchr])
							SNP1maps[chr_indexofchr] = comp(SNP1maps[chr_indexofchr])
							SNP2maps[chr_indexofchr] = reverse(SNP2maps[chr_indexofchr])
							SNP2maps[chr_indexofchr] = comp(SNP2maps[chr_indexofchr])
						}
					}
					//注意：暂时先跳过-1的
					//					if indexofisomaps[chr_indexofchr] == -1 {
					//						continue
					//					}
					//					if indexofisomaps[chr_indexofchr]-1+len(SNP1maps[chr_indexofchr]) > len(Isofastamaps[sam_deal_2lineblocks[0]]) {
					//						L_RAW_SNPmaps[chr_indexofchr] = Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]-1:]
					//					} else {
					//						L_RAW_SNPmaps[chr_indexofchr] = Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]-1 : indexofisomaps[chr_indexofchr]-1+len(SNP1maps[chr_indexofchr])]
					//					}
					//					if indexofisomaps[chr_indexofchr]-1+len(SNP2maps[chr_indexofchr]) > len(Isofastamaps[sam_deal_2lineblocks[0]]) {
					//						S_RAW_SNPmaps[chr_indexofchr] = Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]-1:]

					//					} else {
					//						S_RAW_SNPmaps[chr_indexofchr] = Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]-1 : indexofisomaps[chr_indexofchr]-1+len(SNP2maps[chr_indexofchr])]
					//					}

				}
			}

			if len(Iso_chrmaps) == 0 {
				continue
			} else if len(Iso_chrmaps) == 1 {
				for chr_indexofchr, _ := range Iso_chrmaps {
					indexofiso := strconv.Itoa(indexofisomaps[chr_indexofchr])
					//先输出SNP1的
					_, output1err := output1file.WriteString(sam_deal_2lineblocks[0] + "_" + indexofiso + "_" + SNP1maps[chr_indexofchr] + "\t" + sam_deal_2lineblocks[0] + "\t" + chr_indexofchr + "_" + SNP1maps[chr_indexofchr] + "\t" + indexofiso + "_" + SNP1maps[chr_indexofchr] + "\n")
					//再输出SNP2的
					_, output1err = output1file.WriteString(sam_deal_2lineblocks[0] + "_" + indexofiso + "_" + SNP2maps[chr_indexofchr] + "\t" + sam_deal_2lineblocks[0] + "\t" + chr_indexofchr + "_" + SNP2maps[chr_indexofchr] + "\t" + indexofiso + "_" + SNP2maps[chr_indexofchr] + "\n")
					if output1err == io.EOF {
						fmt.Println("写入错误")
						return
					}
					//注意：暂时先跳过-1的
					if indexofisomaps[chr_indexofchr] == -1 {
						continue
					}
					//先输出SNP1的fasta序列
					_, output2err := output2file.WriteString(">" + sam_deal_2lineblocks[0] + "\t" + indexofiso + "_" + SNP1maps[chr_indexofchr] + "\n")
					_, output2err = output2file.WriteString(Isofastamaps[sam_deal_2lineblocks[0]][:indexofisomaps[chr_indexofchr]-1] + "[" + SNP1maps[chr_indexofchr] + "]" + Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]:] + "\n")

					//再输出SNP2的fasta序列
					_, output2err = output2file.WriteString(">" + sam_deal_2lineblocks[0] + "\t" + indexofiso + "_" + SNP2maps[chr_indexofchr] + "\n")
					_, output2err = output2file.WriteString(Isofastamaps[sam_deal_2lineblocks[0]][:indexofisomaps[chr_indexofchr]-1] + "[" + SNP2maps[chr_indexofchr] + "]" + Isofastamaps[sam_deal_2lineblocks[0]][indexofisomaps[chr_indexofchr]:] + "\n")

					if output2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
			} else if len(Iso_chrmaps) >= 2 {
				FAM := make(map[string]int)
				elem := make(map[int]string)
				var chr_indexofchrslice, IDchr_indexofchrslices []string

				//				fmt.Printf("%s\n", sam_deal_2lineblocks[0])

				for chr_indexofchr, _ := range Iso_chrmaps {

					chr_indexofchrslice = append(chr_indexofchrslice, chr_indexofchr)
				}
				//				fmt.Printf("%s\n", chr_indexofchrslice)

				for i, _ := range chr_indexofchrslice { //把能确定有连锁关系的SNP找出来
					if i >= len(chr_indexofchrslice)-1 {
						break
					}
					for j := i + 1; j < len(chr_indexofchrslice); j++ {
						_, ok1 := phased_rawmaps[chr_indexofchrslice[i]+"|"+chr_indexofchrslice[j]]
						_, ok2 := phased_rawmaps[chr_indexofchrslice[j]+"|"+chr_indexofchrslice[i]]
						if ok1 {
							IDchr_indexofchrslices = append(IDchr_indexofchrslices, chr_indexofchrslice[i]+"|"+chr_indexofchrslice[j])

						} else if ok2 {
							IDchr_indexofchrslices = append(IDchr_indexofchrslices, chr_indexofchrslice[j]+"|"+chr_indexofchrslice[i])
						}
					}
				}
				if len(IDchr_indexofchrslices) < 1 {
					continue
				}
				var i int
				for _, IDchr_indexofchrslice := range IDchr_indexofchrslices { //聚类：找一条iso里所能连锁到最长的链
					IDchr_indexofchrblock := strings.Split(IDchr_indexofchrslice, "|")
					_, ok1 := FAM[IDchr_indexofchrblock[0]]
					_, ok2 := FAM[IDchr_indexofchrblock[1]]
					if !ok1 && !ok2 {
						i++
						FAM[IDchr_indexofchrblock[0]] = i
						FAM[IDchr_indexofchrblock[1]] = i
						elem[i] = IDchr_indexofchrblock[0] + "|" + IDchr_indexofchrblock[1]
					} else if ok1 && ok2 && FAM[IDchr_indexofchrblock[0]] == FAM[IDchr_indexofchrblock[1]] {
						continue
					} else if !ok1 && ok2 {
						FAM[IDchr_indexofchrblock[0]] = FAM[IDchr_indexofchrblock[1]]
						elem[FAM[IDchr_indexofchrblock[1]]] = elem[FAM[IDchr_indexofchrblock[1]]] + "|" + IDchr_indexofchrblock[0]
					} else if ok1 && !ok2 {
						FAM[IDchr_indexofchrblock[1]] = FAM[IDchr_indexofchrblock[0]]
						elem[FAM[IDchr_indexofchrblock[0]]] = elem[FAM[IDchr_indexofchrblock[0]]] + "|" + IDchr_indexofchrblock[1]
					} else if ok1 && ok2 && FAM[IDchr_indexofchrblock[0]] != FAM[IDchr_indexofchrblock[1]] {
						gene2FAM := FAM[IDchr_indexofchrblock[1]] //先把FAM[IDchr_indexofchrblock[1]] 赋给一个变量，以便后面delete
						elem[FAM[IDchr_indexofchrblock[1]]] = seqTrim(elem[FAM[IDchr_indexofchrblock[1]]])
						gene2block := strings.Split(elem[FAM[IDchr_indexofchrblock[1]]], "|")
						for _, gene := range gene2block {
							FAM[gene] = FAM[IDchr_indexofchrblock[0]]
							elem[FAM[IDchr_indexofchrblock[0]]] = elem[FAM[IDchr_indexofchrblock[0]]] + "|" + gene
						}
						delete(elem, gene2FAM)
					}
				}

				var max_SNPlines []string
				for _, V := range elem {
					countsSNP := strings.Split(V, "|")
					if len(countsSNP) > len(max_SNPlines) {
						max_SNPlines = strings.Split(V, "|")
					}
				}

				FAM1 := make(map[string]int)
				elem1 := make(map[int]string)
				var FAMnums int
				for i, _ := range max_SNPlines { //通过聚类的方法将其分为两条iso
					if i >= len(max_SNPlines)-1 {
						break
					}
					for j := i + 1; j < len(max_SNPlines); j++ {
						_, ok1 := phased_rawmaps[max_SNPlines[i]+"|"+max_SNPlines[j]]
						_, ok2 := phased_rawmaps[max_SNPlines[j]+"|"+max_SNPlines[i]]
						if ok1 {
							for _, SNPlinks := range phased_rawmaps[max_SNPlines[i]+"|"+max_SNPlines[j]] {
								SNP1 := strings.Split(SNPlinks, "|")[0]
								SNP2 := strings.Split(SNPlinks, "|")[1]
								if sam_deal_2lineblocks[1] == "16" {
									SNP1 = reverse(SNP1)
									SNP1 = comp(SNP1)
									SNP2 = reverse(SNP2)
									SNP2 = comp(SNP2)
								}
								chr_SNP1 := max_SNPlines[i] + "_" + SNP1
								chr_SNP2 := max_SNPlines[j] + "_" + SNP2
								_, ok3 := FAM1[chr_SNP1]
								_, ok4 := FAM1[chr_SNP2]
								if !ok3 && !ok4 {
									FAMnums++
									FAM1[chr_SNP1] = FAMnums
									FAM1[chr_SNP2] = FAMnums
									elem1[FAMnums] = chr_SNP1 + "|" + chr_SNP2
								} else if ok3 && ok4 && FAM1[chr_SNP1] == FAM1[chr_SNP2] {
									continue
								} else if !ok3 && ok4 {
									FAM1[chr_SNP1] = FAM1[chr_SNP2]
									elem1[FAM1[chr_SNP2]] = elem1[FAM1[chr_SNP2]] + "|" + chr_SNP1
								} else if ok3 && !ok4 {
									FAM1[chr_SNP2] = FAM1[chr_SNP1]
									elem1[FAM1[chr_SNP1]] = elem1[FAM1[chr_SNP1]] + "|" + chr_SNP2
								} else if ok3 && ok4 && FAM1[chr_SNP1] != FAM1[chr_SNP2] {
									FAM_needtodelte := FAM1[chr_SNP2]
									chr_SNPblocks := strings.Split(elem1[FAM1[chr_SNP2]], "|")
									for _, chr_SNPblock := range chr_SNPblocks {
										FAM1[chr_SNPblock] = FAM1[chr_SNP1]
										elem1[FAM1[chr_SNP1]] = elem1[FAM1[chr_SNP1]] + "|" + chr_SNPblock
									}
									delete(elem1, FAM_needtodelte)
								}
							}
						} else if ok2 {
							for _, SNPlinks := range phased_rawmaps[max_SNPlines[j]+"|"+max_SNPlines[i]] {
								SNP1 := strings.Split(SNPlinks, "|")[0]
								SNP2 := strings.Split(SNPlinks, "|")[1]
								if sam_deal_2lineblocks[1] == "16" {
									SNP1 = reverse(SNP1)
									SNP1 = comp(SNP1)
									SNP2 = reverse(SNP2)
									SNP2 = comp(SNP2)
								}
								chr_SNP1 := max_SNPlines[j] + "_" + SNP1
								chr_SNP2 := max_SNPlines[i] + "_" + SNP2
								_, ok3 := FAM1[chr_SNP1]
								_, ok4 := FAM1[chr_SNP2]
								if !ok3 && !ok4 {
									FAMnums++
									FAM1[chr_SNP1] = FAMnums
									FAM1[chr_SNP2] = FAMnums
									elem1[FAMnums] = chr_SNP1 + "|" + chr_SNP2
								} else if ok3 && ok4 && FAM1[chr_SNP1] == FAM1[chr_SNP2] {
									continue
								} else if !ok3 && ok4 {
									FAM1[chr_SNP1] = FAM1[chr_SNP2]
									elem1[FAM1[chr_SNP2]] = elem1[FAM1[chr_SNP2]] + "|" + chr_SNP1
								} else if ok3 && !ok4 {
									FAM1[chr_SNP2] = FAM1[chr_SNP1]
									elem1[FAM1[chr_SNP1]] = elem1[FAM1[chr_SNP1]] + "|" + chr_SNP2
								} else if ok3 && ok4 && FAM1[chr_SNP1] != FAM1[chr_SNP2] {
									FAM_needtodelte := FAM1[chr_SNP2]
									chr_SNPblocks := strings.Split(elem1[FAM1[chr_SNP2]], "|")
									for _, chr_SNPblock := range chr_SNPblocks {
										FAM1[chr_SNPblock] = FAM1[chr_SNP1]
										elem1[FAM1[chr_SNP1]] = elem1[FAM1[chr_SNP1]] + "|" + chr_SNPblock
									}
									delete(elem1, FAM_needtodelte)
								}
							}

						}
					}
				}

				if len(max_SNPlines) > 0 {
					//这里还有修改得余地
					if len(elem1) != 2 {
						continue
					}
					fmt.Printf("%s\n", sam_deal_2lineblocks[0])
					for i, _ := range elem1 {
						indexchangemaps := make(map[int]string)
						var linkofindexoniso string
						SNPlinksblocks := strings.Split(elem1[i], "|")
						for _, SNPlinksblock := range SNPlinksblocks {
							blocks := strings.Split(SNPlinksblock, "_")
							linkofindexoniso = linkofindexoniso + "|" + strconv.Itoa(indexofisomaps[blocks[0]+"_"+blocks[1]]) + "_" + blocks[2]
							indexchangemaps[indexofisomaps[blocks[0]+"_"+blocks[1]]-1] = "[" + blocks[2] + "]"
						}
						linkofindexoniso = linkofindexoniso[1:]
						_, output1err := output1file.WriteString(sam_deal_2lineblocks[0] + "_" + linkofindexoniso + "\t" + sam_deal_2lineblocks[0] + "\t" + elem1[i] + "\t" + linkofindexoniso + "\n")
						if output1err == io.EOF {
							fmt.Println("写入错误")
							return
						}
						_, output2err := output2file.WriteString(">" + sam_deal_2lineblocks[0] + "\t" + linkofindexoniso + "\n")
						if output2err == io.EOF {
							fmt.Println("写入错误")
							return
						}
						for i, r := range Isofastamaps[sam_deal_2lineblocks[0]] {
							_, ok := indexchangemaps[i]
							if ok {
								output2file.WriteString(indexchangemaps[i])
							} else if !ok {
								output2file.WriteString(string(r))
							}
						}
						output2file.WriteString("\n")
						fmt.Printf("%s\n", elem1[i])
						fmt.Printf("%s\n", linkofindexoniso)
					}

				}
			}
		}
		if readsam_deal_2_Err == io.EOF {
			break
		}
	}

}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
func reverse(s string) string {
	r := []rune(s)
	for from, to := 0, len(r)-1; from < to; from, to = from+1, to-1 {
		r[from], r[to] = r[to], r[from]
	}
	return string(r)
}

func comp(s string) string {
	s = strings.Replace(s, "A", "X", -1)
	s = strings.Replace(s, "T", "A", -1)
	s = strings.Replace(s, "X", "T", -1)
	s = strings.Replace(s, "C", "X", -1)
	s = strings.Replace(s, "G", "C", -1)
	s = strings.Replace(s, "X", "G", -1)
	s = strings.Replace(s, "a", "x", -1)
	s = strings.Replace(s, "t", "a", -1)
	s = strings.Replace(s, "x", "t", -1)
	s = strings.Replace(s, "c", "x", -1)
	s = strings.Replace(s, "g", "c", -1)
	s = strings.Replace(s, "x", "g", -1)
	return s
}
