//phasing.go: to phase modified_long_read sequences
//USAGE: ./phasing		modified.longReads.SNP.annotation.txt	chr_index_SNP	SNP_marked.NGS_corrected.modified.longReads.fasta
/*
  AUTOMATIC_OUTPUT: 
  (1) to_be_phased.txt  (clear SNP composition for at least two positions in a single read);
  (2) to_be_phased_2.txt (converting 'to_be_phased.txt' into SNP-SNP combinations);
  (3) to_be_phased_2_ann.txt (revising the format of 'to_be_phased_2.txt');
  (4) to_be_phased_2_stat.txt (summarizing the read frequency of SNP combination given in 'to_be_phased_2_ann.txt');
  (5) phased_raw.txt (all phased results)
  (6) phased_correction.txt (correcting unclear SNP composition with 'phased_raw.txt')
  (7) read_phased.txt (all reads with multiple clear SNPs phased)
  (8) corrected.modified.longReads.fasta (converting clear SNP composition to upcase; otherwise to lowcase, in 'SNP_marked.NGS_corrected.modified.longReads.fasta')
  (9) phased.modified.longReads.fasta (retrieving reads with only one SNP that is clear or with multiple clear SNPs)
*/

//refer to 'phasing.txt'

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"sort"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) <= 2 {
		fmt.Printf("There is no input file!\n")
		os.Exit(1)
	}
	chr_index_SNPfilename := os.Args[2]
	chr_index_SNPfile, open_chr_index_SNPfileErr := os.Open(chr_index_SNPfilename)
	if open_chr_index_SNPfileErr != nil {
		fmt.Printf("Errors happened in opening open_chr_index_SNPfile!\n")
		return
	}
	defer chr_index_SNPfile.Close()
	chr_index_SNPReader := bufio.NewReader(chr_index_SNPfile)
	chr_index_SNPmaps := make(map[string]string)
	for {
		chr_index_SNPraw, readchr_index_SNPErr := chr_index_SNPReader.ReadString('\n')
		chr_index_SNPraw = seqTrim(chr_index_SNPraw)
		if len(chr_index_SNPraw) > 1 {
			lineblocks := strings.Split(chr_index_SNPraw, "\t")
			chr_index_SNPmaps[lineblocks[0]] = lineblocks[1]
		}
		if readchr_index_SNPErr == io.EOF {
			break
		}
	}

	//phasing第一步：打开pseudo_codes_output2文件
	pseudo_codes_out2name := os.Args[1]
	pseudo_codes_out2File, openpseudo_codes_out2Err := os.Open(pseudo_codes_out2name)
	if openpseudo_codes_out2Err != nil {
		fmt.Printf("Errors happened in openingpseudo_codes_out2File!\n")
		return
	}
	defer pseudo_codes_out2File.Close()
	pseudo_codes_out2Reader := bufio.NewReader(pseudo_codes_out2File)

	//创建to_be_phased.txt输出文件
	to_be_phasedfile, _ := os.Create("to_be_phased.txt")
	defer to_be_phasedfile.Close()
	for {
		out2String, readout2Err := pseudo_codes_out2Reader.ReadString('\n')
		out2String = seqTrim(out2String)
		if len(out2String) > 1 {
			lineblocks := strings.Split(out2String, "\t")
			if strings.Contains(lineblocks[2], "|") {
				SNPblocks := strings.Split(lineblocks[2], "|")
				var SNPslices []string
				for _, SNPblock := range SNPblocks {
					if SNPblock[len(SNPblock)-1:len(SNPblock)] == "1" {
						SNPslices = append(SNPslices, SNPblock)
					}
				}
				if len(SNPslices) > 1 {
					for i, SNPslice := range SNPslices {
						if i > 0 {
							_, to_be_phasederr := to_be_phasedfile.WriteString("|")
							if to_be_phasederr == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						_, to_be_phasederr := to_be_phasedfile.WriteString(SNPslice)
						if to_be_phasederr == io.EOF {
							fmt.Println("写入错误")
							return
						}

					}
					_, to_be_phasederr := to_be_phasedfile.WriteString("\n")
					if to_be_phasederr == io.EOF {
						fmt.Println("写入错误")
						return
					}

				}

			} else if !strings.Contains(lineblocks[2], "|") {
				continue
			}
		}
		if readout2Err == io.EOF {
			break
		}
	}

	//phasing第二步：打开to_be_phased文件
	to_be_phasedopenfile, opento_be_phasedErr := os.Open("to_be_phased.txt")
	if opento_be_phasedErr != nil {
		fmt.Printf("Errors happened in opening to_be_phasedopenfile!\n")
		return
	}
	defer to_be_phasedopenfile.Close()
	to_be_phasedReader := bufio.NewReader(to_be_phasedopenfile)

	//创建to_be_phased2.txt输出文件
	to_be_phased2file, _ := os.Create("to_be_phased2.txt")
	defer to_be_phased2file.Close()

	for {
		to_be_phasedString, readto_be_phasedErr := to_be_phasedReader.ReadString('\n')
		to_be_phasedString = seqTrim(to_be_phasedString)
		if len(to_be_phasedString) > 1 {
			lineblocks := strings.Split(to_be_phasedString, "|")
			for i, _ := range lineblocks {
				if i == len(lineblocks)-1 {
					break
				}
				for j := i + 1; j < len(lineblocks); j++ {
					_, to_be_phased2err := to_be_phased2file.WriteString(lineblocks[i] + "|" + lineblocks[j] + "\n")
					if to_be_phased2err == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}

			}

		}
		if readto_be_phasedErr == io.EOF {
			break
		}
	}

	//phasing第三步：打开to_be_phased2文件
	to_be_phased2openfile, opento_be_phased2Err := os.Open("to_be_phased2.txt")
	if opento_be_phased2Err != nil {
		fmt.Printf("Errors happened in opening to_be_phased2openfile!\n")
		return
	}
	defer to_be_phased2openfile.Close()
	to_be_phased2Reader := bufio.NewReader(to_be_phased2openfile)

	//创建to_be_phased_2_ann.txt输出文件
	to_be_phased_2_annfile, _ := os.Create("to_be_phased_2_ann.txt")
	defer to_be_phased_2_annfile.Close()
	for {
		to_be_phased2String, readto_be_phased2Err := to_be_phased2Reader.ReadString('\n')
		to_be_phased2String = seqTrim(to_be_phased2String)
		if len(to_be_phased2String) > 1 {
			var chr_index []string
			var bp []string
			lineblocks := strings.Split(to_be_phased2String, "|")
			for _, lineblock := range lineblocks {
				SNPblocks := strings.Split(lineblock, "_")
				chr_index = append(chr_index, SNPblocks[0]+"_"+SNPblocks[1])
				bp = append(bp, SNPblocks[2])
			}
			for i, _ := range chr_index {
				if i > 0 {
					_, writeto_be_phased_2_annErr := to_be_phased_2_annfile.WriteString("|")
					if writeto_be_phased_2_annErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
				_, writeto_be_phased_2_annErr := to_be_phased_2_annfile.WriteString(chr_index[i])
				if writeto_be_phased_2_annErr == io.EOF {
					fmt.Println("写入错误")
					return
				}
			}
			_, writeto_be_phased_2_annErr := to_be_phased_2_annfile.WriteString("\t")
			if writeto_be_phased_2_annErr == io.EOF {
				fmt.Println("写入错误")
				return
			}
			for i, _ := range bp {
				if i > 0 {
					_, writeto_be_phased_2_annErr := to_be_phased_2_annfile.WriteString("|")
					if writeto_be_phased_2_annErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
				_, writeto_be_phased_2_annErr := to_be_phased_2_annfile.WriteString(bp[i])
				if writeto_be_phased_2_annErr == io.EOF {
					fmt.Println("写入错误")
					return
				}
			}
			_, writeto_be_phased_2_annErr = to_be_phased_2_annfile.WriteString("\n")
			if writeto_be_phased_2_annErr == io.EOF {
				fmt.Println("写入错误")
				return
			}
		}
		if readto_be_phased2Err == io.EOF {
			break
		}
	}

	//phasing第四步：打开to_be_phased_2_ann.txt
	to_be_phased_2_annopenfile, opento_be_phased_2_annErr := os.Open("to_be_phased_2_ann.txt")
	if opento_be_phased_2_annErr != nil {
		fmt.Printf("Errors happened in opening to_be_phased_2_annopenfile!\n")
		return
	}
	defer to_be_phased_2_annopenfile.Close()
	to_be_phased_2_annReader := bufio.NewReader(to_be_phased_2_annopenfile)
	//创建to_be_phased_2_stat.txt文件
	to_be_phased_2_statfile, _ := os.Create("to_be_phased_2_stat.txt")
	defer to_be_phased_2_statfile.Close()
	map1 := make(map[string]int)
	map2 := make(map[string]string)
	map3 := make(map[string]int)
	for {
		to_be_phased_2_annString, to_be_phased_2_annErr := to_be_phased_2_annReader.ReadString('\n')
		to_be_phased_2_annString = seqTrim(to_be_phased_2_annString)
		if len(to_be_phased_2_annString) > 1 {
			lineblocks := strings.Split(to_be_phased_2_annString, "\t")
			_, ok1 := map1[lineblocks[0]]
			_, ok2 := map2[lineblocks[0]]
			_, ok3 := map3[to_be_phased_2_annString]
			if ok1 {
				map1[lineblocks[0]] = map1[lineblocks[0]] + 1
			} else if !ok1 {
				map1[lineblocks[0]] = 1
			}
			if !ok2 {
				map2[lineblocks[0]] = lineblocks[1]
			} else if ok2 {
				flag := 1
				SNP := strings.Split(map2[lineblocks[0]], ",")
				for i, _ := range SNP {
					if lineblocks[1] == SNP[i] {
						flag = 0
						break
					}
				}
				if flag == 1 {
					map2[lineblocks[0]] = map2[lineblocks[0]] + "," + lineblocks[1]
				}
			}
			if ok3 {
				map3[to_be_phased_2_annString] = map3[to_be_phased_2_annString] + 1
			} else if !ok3 {
				map3[to_be_phased_2_annString] = 1
			}

		}
		if to_be_phased_2_annErr == io.EOF {
			break
		}
	}
	for chr_index, SNPs := range map2 {
		SNP := strings.Split(SNPs, ",")
		if len(SNP) == 1 {
			_, writeto_be_phased_2_statErr := to_be_phased_2_statfile.WriteString(chr_index + "\t" + SNPs + "\t" + strconv.Itoa(map3[chr_index+"\t"+SNPs]) + "\t" + "1")
			if writeto_be_phased_2_statErr == io.EOF {
				fmt.Println("写入错误")
				return
			}
		} else if len(SNP) != 1 {
			var frequency string
			for i, _ := range SNP {
				if i > 0 {
					frequency = frequency + ","
				}
				frequency = frequency + strconv.Itoa(map3[chr_index+"\t"+SNP[i]])
			}
			_, writeto_be_phased_2_statErr := to_be_phased_2_statfile.WriteString(chr_index + "\t" + SNPs + "\t" + frequency + "\t" + strconv.Itoa(len(SNP)))
			if writeto_be_phased_2_statErr == io.EOF {
				fmt.Println("写入错误")
				return
			}
		}
		_, writeto_be_phased_2_statErr := to_be_phased_2_statfile.WriteString("\n")
		if writeto_be_phased_2_statErr == io.EOF {
			fmt.Println("写入错误")
			return
		}
	}

	//phasing第五步：打开to_be_phased_2_stat.txt文件
	to_be_phased_2_statopenfile, opento_be_phased_2_statErr := os.Open("to_be_phased_2_stat.txt")
	if opento_be_phased_2_statErr != nil {
		fmt.Printf("Errors happened in opening to_be_phased_2_statopenfile!\n")
		return
	}
	defer to_be_phased_2_statopenfile.Close()
	to_be_phased_2_statReader := bufio.NewReader(to_be_phased_2_statopenfile)
	//创建phased_raw.txt文件
	phased_rawfile, _ := os.Create("phased_raw.txt")
	defer phased_rawfile.Close()
	for {
		to_be_phased_2_statString, to_be_phased_2_statErr := to_be_phased_2_statReader.ReadString('\n')
		to_be_phased_2_statString = seqTrim(to_be_phased_2_statString)
		if len(to_be_phased_2_statString) > 1 {
			lineblocks := strings.Split(to_be_phased_2_statString, "\t")
			if len(lineblocks) < 2 {
				continue
			} else if lineblocks[3] == "2" {
				SNPlink := strings.Split(lineblocks[1], ",")
				SNPlink1 := strings.Split(SNPlink[0], "|")
				SNPlink2 := strings.Split(SNPlink[1], "|")
				if SNPlink1[0] != SNPlink2[0] && SNPlink1[1] != SNPlink2[1] {
					_, writephased_rawfileErr := phased_rawfile.WriteString(to_be_phased_2_statString + "\n")
					if writephased_rawfileErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
			} else if lineblocks[3] == "1" {
				lineblock0 := strings.Split(lineblocks[0], "|")
				lineblock1 := strings.Split(lineblocks[1], "|")
				flag := 1
				var newSNPs string
				for i := 0; i <= 1; i++ {
					if strings.Count(chr_index_SNPmaps[lineblock0[i]], ",") >= 2 {
						_, writephased_rawfileErr := phased_rawfile.WriteString(to_be_phased_2_statString + "\n")
						if writephased_rawfileErr == io.EOF {
							fmt.Println("写入错误")
							return
						}
						flag = 0
						break
					}
					SNPs := strings.Split(chr_index_SNPmaps[lineblock0[i]], ",")
					for _, SNP := range SNPs {
						if SNP != lineblock1[i] {
							newSNPs = newSNPs + "|" + SNP
						}
					}
				}
				if flag == 1 {
					_, writephased_rawfileErr := phased_rawfile.WriteString(lineblocks[0] + "\t" + lineblocks[1] + "," + newSNPs[1:] + "\t" + lineblocks[2] + ",0" + "\t2\n")
					if writephased_rawfileErr == io.EOF {
						fmt.Println("写入错误")
						return
					}

				}
			} else {
				comparemaps := make(map[string]int)
				lineblock1 := strings.Split(lineblocks[1], ",")
				lineblock2 := strings.Split(lineblocks[2], ",")
				var minscore, secondscore int
				var scores []int
				for i, _ := range lineblock1 {
					frequency, _ := strconv.Atoi(lineblock2[i])
					comparemaps[lineblock1[i]] = frequency
					scores = append(scores, frequency)
				}
				sort.Ints(scores)
				minscore = scores[0]
				secondscore = scores[1]
				if secondscore >= minscore*2 {
					for key, score := range comparemaps {
						if score == minscore {
							delete(comparemaps, key)
							break
						}
					}
					_, writephased_rawfileErr := phased_rawfile.WriteString(lineblocks[0] + "\t")
					if writephased_rawfileErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
					lineblock3, _ := strconv.Atoi(lineblocks[3])
					lineblock3 = lineblock3 - 1
					var snp, nums string
					for key, score := range comparemaps {
						snp = snp + "," + key
						nums = nums + "," + strconv.Itoa(score)
					}
					_, writephased_rawfileErr = phased_rawfile.WriteString(snp[1:] + "\t" + nums[1:] + "\t" + strconv.Itoa(lineblock3) + "\n")
					if writephased_rawfileErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
			}

		}
		if to_be_phased_2_statErr == io.EOF {
			break
		}
	}

	//phasing第六步：打开phased_raw.txt文件
	phased_rawopenfile, openphased_rawErr := os.Open("phased_raw.txt")
	if openphased_rawErr != nil {
		fmt.Printf("Errors happened in opening phased_rawopenfile!\n")
		return
	}
	defer phased_rawopenfile.Close()
	phased_rawReader := bufio.NewReader(phased_rawopenfile)
	SNP_Linkagemaps := make(map[string][]string)
	for {
		phased_rawstring, phased_rawErr := phased_rawReader.ReadString('\n')
		phased_rawstring = seqTrim(phased_rawstring)
		if len(phased_rawstring) > 1 {
			lineblocks := strings.Split(phased_rawstring, "\t")
			if lineblocks[3] != "2" {
				continue
			}
			lineblock1 := strings.Split(lineblocks[1], ",")
			for i, _ := range lineblock1 {
				SNP_Linkagemaps[lineblocks[0]] = append(SNP_Linkagemaps[lineblocks[0]], lineblock1[i])
			}
			a := strings.Split(lineblocks[0], "|")
			b := strings.Split(lineblock1[0], "|")
			c := strings.Split(lineblock1[1], "|")
			chrlink := a[1] + "|" + a[0]
			SNPlink1 := b[1] + "|" + b[0]
			SNPlink2 := c[1] + "|" + c[0]

			SNP_Linkagemaps[chrlink] = append(SNP_Linkagemaps[chrlink], SNPlink1, SNPlink2)
		}
		if phased_rawErr == io.EOF {
			break
		}
	}
	//创建phased_correction.txt文件
	phased_correctionfile, _ := os.Create("phased_correction.txt")
	defer phased_correctionfile.Close()
	//再次读取pseudo_codes_out2File
	pseudo_codes_out2File2, openpseudo_codes_out2Err2 := os.Open(pseudo_codes_out2name)
	if openpseudo_codes_out2Err2 != nil {
		fmt.Printf("Errors happened in openingpseudo_codes_out2File2!\n")
		return
	}
	defer pseudo_codes_out2File2.Close()
	pseudo_codes_out2Reader2 := bufio.NewReader(pseudo_codes_out2File2)
	phased_correctionmaps := make(map[string]string)
	for {
		pseudo_codes_out2string, pseudo_codes_out2Err := pseudo_codes_out2Reader2.ReadString('\n')
		pseudo_codes_out2string = seqTrim(pseudo_codes_out2string)
		if len(pseudo_codes_out2string) > 1 {
			lineblocks := strings.Split(pseudo_codes_out2string, "\t")
			if strings.Contains(lineblocks[2], "|") {
				SNPblocks := strings.Split(lineblocks[2], "|")
				var ID_SNPslices []string
				num_0 := 0
				for _, SNPblock := range SNPblocks {
					if SNPblock[len(SNPblock)-1:len(SNPblock)] == "1" {
						ID_SNPslices = append(ID_SNPslices, SNPblock)
					}
					if SNPblock[len(SNPblock)-1:len(SNPblock)] == "0" {
						num_0 = num_0 + 1
					}
				}
				if len(ID_SNPslices) < 1 || num_0 == 0 {
					continue
				}
				for i, SNPblock := range SNPblocks {
					flag := 0
					if SNPblock[len(SNPblock)-1:len(SNPblock)] == "0" {
						for j, _ := range ID_SNPslices {
							no_ID := strings.Split(SNPblocks[i], "_")
							ID := strings.Split(ID_SNPslices[j], "_")
							_, ok := SNP_Linkagemaps[ID[0]+"_"+ID[1]+"|"+no_ID[0]+"_"+no_ID[1]]
							if ok {
								for _, SNPlink := range SNP_Linkagemaps[ID[0]+"_"+ID[1]+"|"+no_ID[0]+"_"+no_ID[1]] {
									singleSNP := strings.Split(SNPlink, "|")
									if singleSNP[0] == ID[2] {
										//三列 － 序列名	在该序列中的SNP顺次	染色体_位置_更正后SNP组成。
										_, writephased_correctionErr := phased_correctionfile.WriteString(lineblocks[0] + "\t" + strconv.Itoa(i+1) + "\t" + no_ID[0] + "_" + no_ID[1] + "_" + singleSNP[1] + "\n")
										if writephased_correctionErr == io.EOF {
											fmt.Println("写入错误")
											return
										}
										phased_correctionmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)] = no_ID[0] + "_" + no_ID[1] + "_" + singleSNP[1] + "_1"
										flag = 1
										break
									}
								}
								if flag == 1 {
									break
								}
							}

						}
					}
				}

			} else if !strings.Contains(lineblocks[2], "|") {
				continue
			}

		}
		if pseudo_codes_out2Err == io.EOF {
			break
		}

	}
	//第7步：再次读取pseudo_codes_out2File
	pseudo_codes_out2File3, openpseudo_codes_out2Err3 := os.Open(pseudo_codes_out2name)
	if openpseudo_codes_out2Err3 != nil {
		fmt.Printf("Errors happened in openingpseudo_codes_out3File2!\n")
		return
	}
	defer pseudo_codes_out2File3.Close()
	pseudo_codes_out2Reader3 := bufio.NewReader(pseudo_codes_out2File3)
	//创建read_phased.txt文件
	read_phasedfile, _ := os.Create("read_phased.txt")
	defer read_phasedfile.Close()
	phasing_SNPmaps := make(map[string]string)
	ninth_outmaps := make(map[string]string)
	for {
		pseudo_codes_out2string, pseudo_codes_out2Err := pseudo_codes_out2Reader3.ReadString('\n')
		pseudo_codes_out2string = seqTrim(pseudo_codes_out2string)
		if len(pseudo_codes_out2string) > 1 {
			lineblocks := strings.Split(pseudo_codes_out2string, "\t")
			if lineblocks[2] == "*" {
				continue
			}
			SNPblocks := strings.Split(lineblocks[2], "|")
			var ID_SNPslices []string
			if strings.Contains(lineblocks[2], "|") {
				for i, SNPblock := range SNPblocks {
					if SNPblock[len(SNPblock)-1:len(SNPblock)] == "1" {
						ID_SNPslices = append(ID_SNPslices, SNPblock)
						phasing_SNPmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)] = SNPblock //第8步校正的map

					} else if SNPblock[len(SNPblock)-1:len(SNPblock)] == "0" {
						_, ok := phased_correctionmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)]
						if ok {
							ID_SNPslices = append(ID_SNPslices, phased_correctionmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)])
							phasing_SNPmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)] = phased_correctionmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)]

						} else if !ok {
							phasing_SNPmaps[lineblocks[0]+"\t"+strconv.Itoa(i+1)] = SNPblock
						}
					}
				}
			} else if !strings.ContainsAny(lineblocks[2], "|*") {
				phasing_SNPmaps[lineblocks[0]+"\t"+"1"] = lineblocks[2]
				if lineblocks[2][len(lineblocks[2])-1:len(lineblocks[2])] == "1" {
					ninth_outmaps[lineblocks[0]] = "1"
				}

			}
			if len(ID_SNPslices) >= 2 {
				ninth_outmaps[lineblocks[0]] = "1" //第9步输出的map
				for i, _ := range ID_SNPslices {
					if i == 0 {
						_, write_read_phasedErr := read_phasedfile.WriteString(lineblocks[0] + "\t")
						if write_read_phasedErr == io.EOF {
							fmt.Println("写入错误")
							return
						}
					}
					if i > 0 {
						_, write_read_phasedErr := read_phasedfile.WriteString("|")
						if write_read_phasedErr == io.EOF {
							fmt.Println("写入错误")
							return
						}

					}
					_, write_read_phasedErr := read_phasedfile.WriteString(ID_SNPslices[i])
					if write_read_phasedErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
				_, write_read_phasedErr := read_phasedfile.WriteString("\n")
				if write_read_phasedErr == io.EOF {
					fmt.Println("写入错误")
					return
				}
			}
		}
		if pseudo_codes_out2Err == io.EOF {
			break
		}
	}
	//fmt.Println("第8步开始")
	//第8步：打开pseudo_codes第一个输出文件
	fasta_filename := os.Args[3]
	fasta_file, openfasta_fileErr := os.Open(fasta_filename)
	if openfasta_fileErr != nil {
		fmt.Printf("Errors happened in opening fasta_file!\n")
		return
	}
	defer fasta_file.Close()
	fasta_Reader := bufio.NewReader(fasta_file)
	//创建corrected.modified.longReads.fasta文件
	Corrected_SMS_CCSfile, _ := os.Create("corrected.modified.longReads.fasta")
	defer Corrected_SMS_CCSfile.Close()
	var seqname string
	fastamaps := make(map[string]string)
	for {
		fastaraw, readfastaErr := fasta_Reader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") || readfastaErr == io.EOF {
			if len(seqname) >= 1 {
				_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(">" + seqname + "\n")
				if writeCorrected_SMS_CCSErr == io.EOF {
					fmt.Println("写入错误")
					return
				}
				if !strings.ContainsAny(fastamaps[seqname], "[]") {
					_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(fastamaps[seqname] + "\n")
					if writeCorrected_SMS_CCSErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				} else if strings.ContainsAny(fastamaps[seqname], "[]") {
					flag := 1
					SNP_nums := 0
					for _, r := range fastamaps[seqname] {
						if string(r) == "[" {
							_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(string(r))
							if writeCorrected_SMS_CCSErr == io.EOF {
								fmt.Println("写入错误")
								return
							}
							flag = 0
							SNP_nums++
							//							fmt.Println(seqname + "\t" + strconv.Itoa(SNP_nums))
							//							fmt.Println(phasing_SNPmaps[seqname+"\t"+strconv.Itoa(SNP_nums)])
							SNPinfo := strings.Split(phasing_SNPmaps[seqname+"\t"+strconv.Itoa(SNP_nums)], "_")
							if SNPinfo[3] == "1" {
								_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(strings.ToUpper(SNPinfo[2]))
								if writeCorrected_SMS_CCSErr == io.EOF {
									fmt.Println("写入错误")
									return
								}
							} else if SNPinfo[3] == "0" {
								_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(strings.ToLower(SNPinfo[2]))
								if writeCorrected_SMS_CCSErr == io.EOF {
									fmt.Println("写入错误")
									return
								}

							}
						}
						if flag == 1 {
							_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(string(r))
							if writeCorrected_SMS_CCSErr == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
						if string(r) == "]" {
							flag = 1
							_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString(string(r))
							if writeCorrected_SMS_CCSErr == io.EOF {
								fmt.Println("写入错误")
								return
							}
						}
					}
					_, writeCorrected_SMS_CCSErr := Corrected_SMS_CCSfile.WriteString("\n")
					if writeCorrected_SMS_CCSErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
			}
			if len(fastaraw) > 0 && strings.Contains(fastaraw, ">") {
				seqname = fastaraw[1:]
			}
		} else if len(fastaraw) > 0 {
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}
	}
	//fmt.Println("第9步开始")
	//第9步：打开corrected.modified.longReads.fasta文件
	Corrected_SMS_CCSopenfile, openCorrected_SMS_CCSErr := os.Open("corrected.modified.longReads.fasta")
	if openCorrected_SMS_CCSErr != nil {
		fmt.Printf("Errors happened in opening Corrected_SMS_CCSopenfile!\n")
		return
	}
	defer Corrected_SMS_CCSopenfile.Close()
	Corrected_SMS_CCSReader := bufio.NewReader(Corrected_SMS_CCSopenfile)
	//创建phased.modified.longReads.fasta
	phased_SMS_CCSfile, _ := os.Create("phased.modified.longReads.fasta")
	defer phased_SMS_CCSfile.Close()
	var seqname2 string
	fastamaps2 := make(map[string]string)
	for {
		Corrected_SMS_CCSraw, Corrected_SMS_CCSErr := Corrected_SMS_CCSReader.ReadString('\n')
		Corrected_SMS_CCSraw = seqTrim(Corrected_SMS_CCSraw)
		if strings.Contains(Corrected_SMS_CCSraw, ">") || Corrected_SMS_CCSErr == io.EOF {
			if len(seqname2) > 1 {
				_, ok := ninth_outmaps[seqname2]
				if ok {
					_, writephased_SMS_CCSErr := phased_SMS_CCSfile.WriteString(">" + seqname2 + "\n" + fastamaps2[seqname2] + "\n")
					if writephased_SMS_CCSErr == io.EOF {
						fmt.Println("写入错误")
						return
					}
				}
			}
			if len(Corrected_SMS_CCSraw) > 0 && strings.Contains(Corrected_SMS_CCSraw, ">") {
				seqname2 = Corrected_SMS_CCSraw[1:]
			}
		} else if len(Corrected_SMS_CCSraw) > 0 {
			fastamaps2[seqname2] = fastamaps2[seqname2] + Corrected_SMS_CCSraw
		}

		if Corrected_SMS_CCSErr == io.EOF {
			break
		}

	}
}
func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
