//pseudoReadRefCoord.go: to parse the (forward) reference genome coordinates of each transcript according to the CIGAR information given in 'transcripts.txt' file
//USAGE: ./pseudoReadRefCoord	transcripts.txt		>pseudo.longReads.coord.txt

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) <= 1 { 
		fmt.Printf("There is no input file!\n") 
		os.Exit(1)                              
	}
	inputFileName := os.Args[1]                  
	inputFile, openErr := os.Open(inputFileName) 
	if openErr != nil {
		fmt.Printf("Errors happened in opening the input file!\n")
		return
	}
	defer inputFile.Close()
	inputReader := bufio.NewReader(inputFile)
	for {

		inputString, readErr := inputReader.ReadString('\n')
		inputString = seqTrim(inputString)
		if readErr == io.EOF && len(inputString) < 1 {
			break
		}
		if len(inputString) < 1 {
			continue
		}
		if strings.Contains(inputString, "@HD\t") || strings.Contains(inputString, "@PG\t") || strings.Contains(inputString, "@SQ\t") {
			continue
		}
		inputStringblock := strings.Split(inputString, "\t")
		if inputStringblock[1] == "4" {
			continue
		}

		var all_lines string

		all_lines = inputStringblock[0] + "\t" + inputStringblock[2] + "\t" + inputStringblock[3] + "\t" + inputStringblock[5]
		inputStringblock3, _ := strconv.ParseFloat(inputStringblock[3], 64)

		//		intron部分
		var introns string

		intronblocks := strings.SplitAfter(inputStringblock[5], "N")
		intronmaps := make(map[string]float64)
		var lastNminus, Nminus, startnum float64 = 0, 0, 0
		var endnum float64 = inputStringblock3 - 1

		for _, intronblock := range intronblocks {
			intronadd := 0

			for i, r := range intronblock { //将MIDNSHP=X的位置提取出来
				if strings.ContainsAny(string(r), "MIDNSHP=X") {
					intronnums, _ := strconv.ParseFloat(intronblock[intronadd:i], 64)
					intronmaps[string(r)] = intronmaps[string(r)] + intronnums
					intronadd = i + 1
					if r == 'N' { //要把本'N'的数字值去掉
						Nminus = intronnums
					}
				}
			}
			startnum = (endnum + lastNminus + 1)
			endnum = (inputStringblock3 + intronmaps["M"] + intronmaps["N"] + intronmaps["="] + intronmaps["X"] + intronmaps["D"] + intronmaps["P"] - Nminus - 1)
			lastNminus = Nminus
			Nminus = 0
			introns = introns + "|" + strconv.FormatFloat(startnum, 'f', 0, 64) + "_" + strconv.FormatFloat(endnum, 'f', 0, 64)

		}
		introns = introns[1:]
		all_lines = all_lines + "\t" + introns

		fmt.Printf("%s\n", all_lines)
		//		if !strings.Contains(inputStringblock[5], "N") {
		//			all_lines = all_lines + "\t" + "-"
		//		}

		if readErr == io.EOF {
			break
		}
	}

}
func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
