//pseudoReadSeqExt.go: to build pseudo long read sequences with reference genome sequences and using transcript frames indicated in 'pseudo.longReads.coord.txt'
//USAGE: ./pseudoReadsSeqExt	REF_SEQ		pseudo.longReads.coord.txt		>pseudo.longReads.fasta

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(fastafile or samfile)!\n")
		return
	}
	fastaFilename := os.Args[1]
	fastaFile, openErr1 := os.Open(fastaFilename)
	if openErr1 != nil {
		fmt.Printf("Error happened in opening %s\n", fastaFilename)
		return
	}
	defer fastaFile.Close()

	fastaReader := bufio.NewReader(fastaFile)
	var seq, chr string
	seqmaps := make(map[string][]string)
	for {
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
        fastaraw = strings.Trim(fastaraw,"\n")
        if(strings.Contains(fastaraw,">")){
           chr = strings.Replace(fastaraw,">","",-1)
        } else if len(fastaraw) > 0 {
          seq = fastaraw
          seqmaps[chr] = append(seqmaps[chr],seq)
        }         	

        if readfastaErr == io.EOF {
			break
		}
	}

	coordfilename := os.Args[2]
	coordFile, openErr2 := os.Open(coordfilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", coordfilename)
		return
	}
	defer coordFile.Close()
	coordReader := bufio.NewReader(coordFile)
    var lineStart, lineEnd, posStart, posEnd int
	for {
		coordraw, readcoordErr := coordReader.ReadString('\n')
		coordraw = strings.Trim(coordraw,"\n")
		if len(coordraw) > 0 {
		  lineblocks := strings.Split(coordraw, "\t")
		  fmt.Printf(">%s\n", lineblocks[0])
		  indexblocks := strings.Split(lineblocks[4], "|")
		  for _, indexblock := range indexblocks {
			  index := strings.Split(indexblock, "_")
			  index0, _ := strconv.Atoi(index[0])
			  index1, _ := strconv.Atoi(index[1])
			  if index0 % len(seqmaps[lineblocks[1]][0]) != 0 {
                 posStart = index0 % len(seqmaps[lineblocks[1]][0]) - 1
			     lineStart = (index0 - index0 % len(seqmaps[lineblocks[1]][0]))/len(seqmaps[lineblocks[1]][0])
			  } else {
			     lineStart = (index0 - index0 % len(seqmaps[lineblocks[1]][0]))/len(seqmaps[lineblocks[1]][0]) - 1
			     posStart = len(seqmaps[lineblocks[1]][0]) - 1
			  }
			  if index1 % len(seqmaps[lineblocks[1]][0]) != 0 {
                 posEnd = index1 % len(seqmaps[lineblocks[1]][0])
			     lineEnd = (index1 - index1 % len(seqmaps[lineblocks[1]][0]))/len(seqmaps[lineblocks[1]][0])
			  } else {
			  	 lineEnd = (index1 - index1 % len(seqmaps[lineblocks[1]][0]))/len(seqmaps[lineblocks[1]][0]) - 1
			     posEnd = len(seqmaps[lineblocks[1]][0])
			  }   
			  if lineEnd - lineStart >= 1 {
			     fmt.Printf("%s",seqmaps[lineblocks[1]][lineStart][posStart:len(seqmaps[lineblocks[1]][0])])
			   	 for i:= lineStart+1; i<lineEnd; i++ {
                    fmt.Printf("%s",seqmaps[lineblocks[1]][i][0:len(seqmaps[lineblocks[1]][0])])
			   	 }
			   	 fmt.Printf("%s",seqmaps[lineblocks[1]][lineEnd][0:posEnd])
			  } else {
			     fmt.Printf("%s",seqmaps[lineblocks[1]][lineStart][posStart:posEnd])
			  }		     		
		  }
		  fmt.Printf("\n")
	    }
		if readcoordErr == io.EOF {
			break
		}
	}
}	
