//rawReadSeqModify.go: to modify raw_long_read sequences to the same direction with the forward strand of reference genomes 
//USAGE: ./rawReadSeqModify		transcripts.txt		RAW_LONG_READS.fasta	>modified.longReads.fasta

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}

	sam_dealfilename := os.Args[1]
	sam_dealfile, opensam_dealErr := os.Open(sam_dealfilename)
	if opensam_dealErr != nil {
		fmt.Printf("Error happened in opening %s\n", sam_dealfilename)
		return
	}
	defer sam_dealfile.Close()
	sam_dealReader := bufio.NewReader(sam_dealfile)
	reversemaps := make(map[string]string)
	for {
		sam_dealraw, readsam_dealErr := sam_dealReader.ReadString('\n')
		sam_dealraw = seqTrim(sam_dealraw)
		if len(sam_dealraw) > 0 {
			samlineblocks := strings.Split(sam_dealraw, "\t")
			reversemaps[samlineblocks[0]] = samlineblocks[1]
		}
		if readsam_dealErr == io.EOF {
			break
		}
	}

	fastafilename := os.Args[2]
	fastaFile, openErr2 := os.Open(fastafilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", fastafilename)
		return
	}
	defer fastaFile.Close()
	fastaReader := bufio.NewReader(fastaFile)
	var seqname string
	fastamaps := make(map[string]string)
	for {
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") || readfastaErr == io.EOF {
			if len(seqname) >= 1 {
				_, ok := reversemaps[seqname]
				if ok {
					if reversemaps[seqname] == "0" {
						fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
					} else if reversemaps[seqname] == "16" {
						fastamaps[seqname] = reverse(fastamaps[seqname])
						fastamaps[seqname] = comp(fastamaps[seqname])
						fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
					}
				}
			}
			if len(fastaraw) > 0 && strings.Contains(fastaraw, ">") {
				seqname = fastaraw[1:]
			}
		} else if len(fastaraw) > 0 {
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}
	}

}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
func reverse(s string) string {
	r := []rune(s)
	for from, to := 0, len(r)-1; from < to; from, to = from+1, to-1 {
		r[from], r[to] = r[to], r[from]
	}
	return string(r)
}

func comp(s string) string {
	s = strings.Replace(s, "A", "X", -1)
	s = strings.Replace(s, "T", "A", -1)
	s = strings.Replace(s, "X", "T", -1)
	s = strings.Replace(s, "C", "X", -1)
	s = strings.Replace(s, "G", "C", -1)
	s = strings.Replace(s, "X", "G", -1)
	s = strings.Replace(s, "a", "x", -1)
	s = strings.Replace(s, "t", "a", -1)
	s = strings.Replace(s, "x", "t", -1)
	s = strings.Replace(s, "c", "x", -1)
	s = strings.Replace(s, "g", "c", -1)
	s = strings.Replace(s, "x", "g", -1)
	return s
}
