//snpMark.go:
//USAGE: ./snpMark	FILTERED_VCF_FILE	lq_marked.NGS_corrected.pseudo.longReads.fasta		>SNP_marked.NGS_corrected.pseudo.longReads.fasta

/*
  (1) Homozygous mutations in sampled individual (AC=2;AN=2) – If INDEL, converting the base into X, otherwise substitute it with the corresponding mutation;
  (2) Heterozygous mutations in sampled individual (AC=1;AN=2 OR AC=1,1;AN=2 --- one being identical to reference, or neither being identical to reference), converting the base into 1.
*/

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) < 3 {
		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	VCFFilename := os.Args[1]
	VCFFile, openErr1 := os.Open(VCFFilename)
	if openErr1 != nil {
		fmt.Printf("Error happened in opening %s\n", VCFFilename)
		return
	}
	defer VCFFile.Close()

	VCFReader := bufio.NewReader(VCFFile)
	VCFmaps := make(map[string][]string)
	for {
		VCFraw, readVCFErr := VCFReader.ReadString('\n')
		VCFraw = seqTrim(VCFraw)
		if len(VCFraw) > 1 {
			lineblocks := strings.Split(VCFraw, "\t")
			if strings.Contains(VCFraw, "#") || len(lineblocks) < 2 || strings.Contains(VCFraw, "LOWQUAL") {
				continue
			}
			if strings.Contains(VCFraw, "AC=2;AN=2") {
				info := lineblocks[1] + ";" + lineblocks[3] + ";" + lineblocks[4] + ";" + "2"
				VCFmaps[lineblocks[0]] = append(VCFmaps[lineblocks[0]], info)
			}
			if strings.Contains(VCFraw, "AC=1;AN=2") || strings.Contains(VCFraw, "AC=1,1;AN=2") {
				info := lineblocks[1] + ";" + lineblocks[3] + ";" + lineblocks[4] + ";" + "1"
				VCFmaps[lineblocks[0]] = append(VCFmaps[lineblocks[0]], info)
			}
		}
		if readVCFErr == io.EOF {
			break
		}
	}

	fastafilename := os.Args[2]
	fastaFile, openErr2 := os.Open(fastafilename)
	if openErr2 != nil {
		fmt.Printf("Error happened in opening %s\n", fastafilename)
		return
	}
	defer fastaFile.Close()
	fastaReader := bufio.NewReader(fastaFile)
	var seqname string
	fastamaps := make(map[string]string)

	for {
		fastaraw, readfastaErr := fastaReader.ReadString('\n')
		fastaraw = seqTrim(fastaraw)
		if strings.Contains(fastaraw, ">") {
			if len(seqname) > 1 {
				for _, info := range VCFmaps[seqname] {
					infoblocks := strings.Split(info, ";")
					if infoblocks[3] == "2" {
						index_int, _ := strconv.Atoi(infoblocks[0])
						if len(infoblocks[1]) != len(infoblocks[2]) {
							fastamaps[seqname] = fastamaps[seqname][:index_int-1] + "X" + fastamaps[seqname][index_int:]
						} else if len(infoblocks[1]) == len(infoblocks[2]) && len(infoblocks[1]) == 1 {
							fastamaps[seqname] = fastamaps[seqname][:index_int-1] + infoblocks[2] + fastamaps[seqname][index_int:]
						}

					} else if infoblocks[3] == "1" {
						index_int, _ := strconv.Atoi(infoblocks[0])
						fastamaps[seqname] = fastamaps[seqname][:index_int-1] + "1" + fastamaps[seqname][index_int:]
					}
				}
				fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
			}
			seqname = fastaraw[1:]
		} else if len(fastaraw) > 0 {
			fastamaps[seqname] = fastamaps[seqname] + fastaraw
		}
		if readfastaErr == io.EOF {
			break
		}
	}

	if len(seqname) > 1 {
		for _, info := range VCFmaps[seqname] {
			infoblocks := strings.Split(info, ";")
			if infoblocks[3] == "2" {
				index_int, _ := strconv.Atoi(infoblocks[0])
				if len(infoblocks[1]) != len(infoblocks[2]) {
					fastamaps[seqname] = fastamaps[seqname][:index_int-1] + "X" + fastamaps[seqname][index_int:]
				} else if len(infoblocks[1]) == len(infoblocks[2]) && len(infoblocks[1]) == 1 {
					fastamaps[seqname] = fastamaps[seqname][:index_int-1] + infoblocks[2] + fastamaps[seqname][index_int:]
				}

			} else if infoblocks[3] == "1" {
				index_int, _ := strconv.Atoi(infoblocks[0])
				fastamaps[seqname] = fastamaps[seqname][:index_int-1] + "1" + fastamaps[seqname][index_int:]
			}
		}
		fmt.Printf(">%s\n%s\n", seqname, fastamaps[seqname])
	}
}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
