
//snpSet.go: to annotate the SNP compsition set according to reference genome coordinate
//USAGE: ./snpSet  pseudo.longReads.coord.txt  FILTER_VCF
//AUTOMATIC OUTPUT: chr_index_SNP.txt

package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	chr_index_SNPfile, _ := os.Create("chr_index_SNP.txt")
	defer chr_index_SNPfile.Close()
	if len(os.Args) < 3 {

		fmt.Printf("Lacking of input file(VCFfile or fastafile)!\n")
		return
	}
	sam_deal_2filename := os.Args[1]
	sam_deal_2file, opensam_dealErr := os.Open(sam_deal_2filename)
	if opensam_dealErr != nil {
		fmt.Printf("Error happened in opening %s\n", sam_deal_2filename)
		return
	}
	defer sam_deal_2file.Close()
	sam_dealReader := bufio.NewReader(sam_deal_2file)
	SAM_CIGARmaps := make(map[string]string)
	chr_maps := make(map[string]string)
	chrindex_maps := make(map[string]int)
	for {
		sam_dealraw, readsam_dealErr := sam_dealReader.ReadString('\n')
		sam_dealraw = seqTrim(sam_dealraw)
		if len(sam_dealraw) > 0 {
			samlineblocks := strings.Split(sam_dealraw, "\t")
			SAM_CIGARmaps[samlineblocks[0]] = samlineblocks[3]
			chr_maps[samlineblocks[0]] = samlineblocks[1]
			chrindex, _ := strconv.Atoi(samlineblocks[2])
			chrindex = chrindex - 1
			chrindex_maps[samlineblocks[0]] = chrindex
		}
		if readsam_dealErr == io.EOF {
			break
		}
	}
	fmt.Println("SAM_deal_2 读取结束", len(SAM_CIGARmaps))

	VCFfilename := os.Args[2]
	VCFfile, openVCFErr := os.Open(VCFfilename)
	if openVCFErr != nil {
		fmt.Printf("Error happened in opening %s\n", VCFfilename)
		return
	}
	defer VCFfile.Close()
	VCFReader := bufio.NewReader(VCFfile)
	chr_index_SNPmaps := make(map[string]string)
	for {
		VCFraw, readVCFErr := VCFReader.ReadString('\n')
		VCFraw = seqTrim(VCFraw)
		var pse_readchrindex int
		if len(VCFraw) > 1 {
			VCFlineblocks := strings.Split(VCFraw, "\t")
			if strings.Contains(VCFraw, "#") || len(VCFlineblocks) < 2 || strings.Contains(VCFraw, "LOWQUAL") || strings.Contains(VCFraw, "AC=2") {
				continue
			} else if strings.Contains(VCFraw, "AC=1;AN=2") {
				if strings.Contains(VCFraw, "057/ccs	523	.	T") {
					fmt.Println("找到了")
				}
				pse_readindex, _ := strconv.Atoi(VCFlineblocks[1])
				CIGARmaps := make(map[string]int)
				lenthadd := 0
				for j, s := range SAM_CIGARmaps[VCFlineblocks[0]] {
					if strings.ContainsAny(string(s), "MIDNSHP=X") {
						lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[VCFlineblocks[0]][lenthadd:j])
						CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
						lenthadd = j + 1
						if CIGARmaps["M"]+CIGARmaps["D"] >= pse_readindex {
							pse_readchrindex = pse_readindex + chrindex_maps[VCFlineblocks[0]] + CIGARmaps["N"]
							if strings.Contains(VCFraw, "057/ccs	523	.	T") {
								fmt.Println(chrindex_maps[VCFlineblocks[0]])
							}
							break
						}
					}
				}
				_, ok := chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)]
				if !ok {
					chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] = VCFlineblocks[3] + "," + VCFlineblocks[4]
				} else if ok {
					for i := 3; i <= 4; i++ {
						flag := 1
						SNPs := strings.Split(chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)], ",")
						for j, _ := range SNPs {
							if VCFlineblocks[i] == SNPs[j] {
								flag = 0
								break
							}
						}
						if flag == 1 {
							chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] = chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] + "," + VCFlineblocks[i]
						}
					}
				}
			} else if strings.Contains(VCFraw, "AC=1,1") {
				pse_readindex, _ := strconv.Atoi(VCFlineblocks[1])
				CIGARmaps := make(map[string]int)
				lenthadd := 0
				for j, s := range SAM_CIGARmaps[VCFlineblocks[0]] {
					if strings.ContainsAny(string(s), "MIDNSHP=X") {
						lenthnums, _ := strconv.Atoi(SAM_CIGARmaps[VCFlineblocks[0]][lenthadd:j])
						CIGARmaps[string(s)] = CIGARmaps[string(s)] + lenthnums
						lenthadd = j + 1
						if CIGARmaps["M"]+CIGARmaps["D"] >= pse_readindex {
							pse_readchrindex = pse_readindex + chrindex_maps[VCFlineblocks[0]] + CIGARmaps["N"]
							break
						}
					}
				}
				_, ok := chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)]
				if !ok {
					chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] = VCFlineblocks[4]
				} else if ok {
					VCFlineblocks4 := strings.Split(VCFlineblocks[4], ",")
					for i := 0; i <= 1; i++ {
						flag := 1
						SNPs := strings.Split(chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)], ",")
						for j, _ := range SNPs {
							if VCFlineblocks4[i] == SNPs[j] {
								flag = 0
								break
							}
						}
						if flag == 1 {
							chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] = chr_index_SNPmaps[chr_maps[VCFlineblocks[0]]+"_"+strconv.Itoa(pse_readchrindex)] + "," + VCFlineblocks4[i]
						}
					}
				}
			}
		}
		if readVCFErr == io.EOF {
			break
		}
	}
	fmt.Println("VCF 读取结束")
	for chr, SNP := range chr_index_SNPmaps {
		_, writechr_index_SNPfileErr := chr_index_SNPfile.WriteString(chr + "\t" + SNP + "\n")
		if writechr_index_SNPfileErr == io.EOF {
			fmt.Println("写入错误")
			return
		}
	}
}

func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
