//transcriptIdentify.go: to identify the transcripts and reference genome mapped coordinates of raw long reads according to SAM file.
//USAGE: ./transcriptIdentify	SAM_FILE	>transcripts.txt

//逐行过滤，按照前面所说的两个标准；同时记录符合标准的每个read对应mapping的read名（第1列）、mapping flag（第2列）
//、对映的染色体编号（第3列）、匹配的起始位置（第4列）、匹配的终止位置（起始位置 + "CIGAR中M/D/N/=/X字符前数字"总和 - 1
//）、CIGAR（第6列）、mapLength、mapQuality、intron集合（没有intron，就用“-”符号表示；如果有的话，就用一个字符串来完整表示
//；多个intron之间用“|”号间隔；每个intron用“reference序列中的起始位置_reference序列中的终止位置”来表示；CIGAR字符串中每一个N表示
//一个intron，N前面的数字表示本intron的长度，计算每个N长度数字前的所有“M/D/N/=/X字符前数字”总和 + 匹配起始位置，该值表示本intron起
//始位置；再计算该intron起始位置＋本N前面的数字 - 1，表示该intron的终止位置）。


package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"strconv"
	"strings"
)

func main() {
	if len(os.Args) <= 1 { 
		fmt.Printf("There is no input file!\n") 
		os.Exit(1)                              
	}
	inputFileName := os.Args[1]                  
	inputFile, openErr := os.Open(inputFileName) 
	if openErr != nil {
		fmt.Printf("Errors happened in opening the input file!\n")
		return
	}
	defer inputFile.Close()
	inputReader := bufio.NewReader(inputFile)
	readmap := make(map[string]string)
	for {

		maplenthmaps := make(map[string]float64)
		lenthadd := 0
		inputString, readErr := inputReader.ReadString('\n')
		inputString = seqTrim(inputString)
		if readErr == io.EOF && len(inputString) < 1 {
			break
		}
		if len(inputString) < 1 {
			continue
		}
		if strings.Contains(inputString, "@HD\t") || strings.Contains(inputString, "@PG\t") || strings.Contains(inputString, "@SQ\t") {
			continue
		}
		inputStringblock := strings.Split(inputString, "\t")
		if inputStringblock[1] == "4" {
			continue
		}

		var all_lines string
		var endindex float64 //储存匹配的终止位置
		all_lines = inputStringblock[0] + "\t" + inputStringblock[1] + "\t" + inputStringblock[2] + "\t" + inputStringblock[3]
		inputStringblock3, _ := strconv.ParseFloat(inputStringblock[3], 64)

		//maplenth部分
		for i, r := range inputStringblock[5] {
			if strings.ContainsAny(string(r), "MIDNSHP=X") {
				lenthnums, _ := strconv.ParseFloat(inputStringblock[5][lenthadd:i], 64)
				maplenthmaps[string(r)] = maplenthmaps[string(r)] + lenthnums
				lenthadd = i + 1
			}
		}
		mapLenth := (maplenthmaps["M"] + maplenthmaps["I"]) / (maplenthmaps["M"] + maplenthmaps["I"] + maplenthmaps["S"] + maplenthmaps["="] + maplenthmaps["X"] + maplenthmaps["H"])
		if mapLenth < 0.67 {
			continue
		}

		endindex = inputStringblock3 + (maplenthmaps["M"] + maplenthmaps["D"] + maplenthmaps["N"] + maplenthmaps["="] + maplenthmaps["X"]) - 1
		all_lines = all_lines + "\t" + strconv.FormatFloat(endindex, 'f', 0, 64) + "\t" + inputStringblock[5] + "\t" + strconv.FormatFloat(mapLenth, 'f', -1, 64)

		//mapQualiity部分
		qualityadd := 0
		var qualitynums float64
		for i, _ := range inputStringblock {
			if strings.Contains(inputStringblock[i], "MD:Z:") {
				inputStringblock[i] = inputStringblock[i] + "A"
				for j, r := range inputStringblock[i] {
					if !strings.ContainsAny(string(r), "1234567890") {
						qualitynum, _ := strconv.ParseFloat(inputStringblock[i][qualityadd:j], 64)
						qualitynums = qualitynums + qualitynum
						qualityadd = j + 1
					}
				}
				break
			}
		}
		mapQuality := qualitynums / (maplenthmaps["M"] + maplenthmaps["I"])
		if mapQuality < 0.75 {
			continue
		}
		all_lines = all_lines + "\t" + strconv.FormatFloat(mapQuality, 'f', -1, 64)

		//		intron部分
		var introns string
		if strings.Contains(inputStringblock[5], "N") {
			intronblocks := strings.SplitAfter(inputStringblock[5], "N")
			intronmaps := make(map[string]float64)
			for _, intronblock := range intronblocks {
				intronadd := 0
				var Nminus float64
				if strings.Contains(intronblock, "N") {
					for i, r := range intronblock { //将MIDNSHP=X的位置提取出来
						if strings.ContainsAny(string(r), "MIDNSHP=X") {
							intronnums, _ := strconv.ParseFloat(intronblock[intronadd:i], 64)
							intronmaps[string(r)] = intronmaps[string(r)] + intronnums
							intronadd = i + 1
							if r == 'N' { //要把本'N'的数字值去掉
								Nminus = intronnums
							}
						}
					}
					startnum := (inputStringblock3 + intronmaps["M"] + intronmaps["N"] + intronmaps["="] + intronmaps["X"] + intronmaps["D"] - Nminus)
					endnum := (inputStringblock3 + intronmaps["M"] + intronmaps["N"] + intronmaps["="] + intronmaps["X"] + intronmaps["D"] - 1)
					introns = introns + "|" + strconv.FormatFloat(startnum, 'f', 0, 64) + "_" + strconv.FormatFloat(endnum, 'f', 0, 64)

				}
			}
			introns = introns[1:]
			all_lines = all_lines + "\t" + introns

		}
		if !strings.Contains(inputStringblock[5], "N") {
			all_lines = all_lines + "\t" + "-"
		}

		for i, _ := range inputStringblock {
			if strings.Contains(inputStringblock[i], "NH:i:") {
				readfreq, _ := strconv.ParseFloat(inputStringblock[i][5:], 64)
				if readfreq > 1 {
					all_lines = all_lines + "\t2"
					readmap[inputStringblock[0]] = readmap[inputStringblock[0]] + ";" + all_lines
				}
				if readfreq == 1 {
					all_lines = all_lines + "\t1"
					fmt.Printf("%s\n", all_lines)
				}
				break
			}
		}
		if readErr == io.EOF {
			break
		}
	}

	for readname, _ := range readmap {
		readblocks := strings.Split(readmap[readname], ";")
		if len(readblocks) == 2 {
			fmt.Printf("%s\n", readblocks[1])
			continue
		}
		var maxscore float64 = 0
		var maxindex int
		var secondscore float64 = 0
		//		var secondindex int
		for i, _ := range readblocks { //寻找最大值的循环
			if i == 0 {
				continue
			}
			readblock := strings.Split(readblocks[i], "\t")
			readblock6, _ := strconv.ParseFloat(readblock[6], 64)
			readblock7, _ := strconv.ParseFloat(readblock[7], 64)
			lenthmultquality := readblock6 * readblock7
			if lenthmultquality > maxscore {
				maxscore = lenthmultquality
				maxindex = i
			}
		}
		for i, _ := range readblocks { //寻找第二大循环
			if i == 0 || i == maxindex {
				continue
			}
			readblock := strings.Split(readblocks[i], "\t")
			readblock6, _ := strconv.ParseFloat(readblock[6], 64)
			readblock7, _ := strconv.ParseFloat(readblock[7], 64)
			lenthmultquality := readblock6 * readblock7
			if lenthmultquality <= maxscore && lenthmultquality > secondscore {
				secondscore = lenthmultquality
				//				secondindex = i
			}
		}
		if secondscore < 0.98*maxscore {
			fmt.Printf("%s\n", readblocks[maxindex])
		}

	}
}
func seqTrim(s string) string {
	s = strings.Trim(s, "\r\n")
	s = strings.Trim(s, "\n")
	return s
}
